#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import <SFFiscalPeriodManager.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFAccountingPeriodFunction.h"
#import "SFParameterData.h"
#import "SFFiscalPeriod.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;					/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

static NSString *descriptionText = @"AccountingPeriod function returns the starting and ending "
							@"dates of the fiscal period. The fiscal period is passed to the "
							@"function in YY/PPP format where YY is the fiscal year, "
							@"and PPP is the number of period with leading zeros. For "
							@"example, period 2 of fiscal year 1994 that has total of 12 "
							@"periods would be formatted as 94/02.\n\n"
							@"The function takes the following input parameters:\n"
							@"<Period> - fiscal period in YY/PPP format\n"
							@"<Start> - starting date of the period, in YYYYMMDD format\n"
							@"<End> - ending date of the period, in YYYYMMDD format\n";

@implementation SFAccountingPeriodFunction : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	self = [super  initFor:sender];
	fiscalPeriodManager = [[SFFiscalPeriodManager  alloc]  init];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Period"
						dataType:ACCPERIOD
						displayedDataType:ACCPERIOD
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Start date"
						dataType:DATE
						displayedDataType:DATE
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:NO];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"End date"
						dataType:DATE
						displayedDataType:DATE
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:NO];
	[parameterList  addObjectWithoutRetaining:parameter];
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (UNKNOWN);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (descriptionText);
}

- (NSString *) functionName
{
	return (@"AccountingPeriod");
}

- execute: (NSArray *) parmList
{
	NSString *period;
	SFFiscalPeriod *periodData;
	[super  execute:nil];
	period = [[parmList  objectAtIndex:0]  asciiValue];
	fiscalPeriodManager = [SFFiscalPeriodManager  instance];
	if (fiscalPeriodManager == nil)
		fiscalPeriodManager = [[SFFiscalPeriodManager  alloc]  init];
	periodData = [fiscalPeriodManager  nearestDataForPeriodString:period];
	[[parmList  objectAtIndex:1]  setValue:[periodData  startDate]];
	[[parmList  objectAtIndex:2]  setValue:[periodData  endDate]];
	return nil;
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
	[parameterList  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end