#import <libtools.h>
#import <DefinedConstants.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFAddToDate.h"
#import "SFStringTableManager.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;					/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */
static int daysInMonth[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
NSString *addToDate(NSString *date, int nDays);

@implementation SFAddToDate : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	self = [super  initFor:sender];
	stringTableManager = [[SFStringTableManager  alloc]  init];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:DATE];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Date"
						dataType:DATE
						displayedDataType:DATE
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Increment"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Units"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameter  setStringTable:[stringTableManager  dateUnitList]];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (DATE);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"AddToDate function increments or decrements the date by the given number; "
		    @"the number is treated as days, weeks, months or years according to the setting "
		    @"of Units parameter."
		    @"\n\nThe function takes the following input parameters:\n"
		    @"<Date> - original date\n"
		    @"<Increment> - number of date units to be added (if positive) to or subtracted "
		    @"(if negative) from the original date\n"
		    @"<Units> - date units\n");
}

- (NSString *) functionName
{
	return (@"AddToDate");
}

- execute: (NSArray *) parmList
{
	NSString *date, *units;
	int increment, day, month, year, nYears;
	date = [[parmList  objectAtIndex:0]  value];
	increment = [[parmList  objectAtIndex:1]  intValue];
	units = [[parmList  objectAtIndex:2]  value];
	if (stringsAreEqual(units,@"Days"))
		date =  addToDate(date,increment);
	else if (stringsAreEqual(units,@"Weeks"))
		date =  addToDate(date,increment * 7);
	else if (stringsAreEqual(units,@"Months"))
	{
		sscanf([date  cString],"%4d%2d%2d",&year,&month,&day);
		if (abs(increment) > 12)
		{
			if (increment > 0)
			{
				nYears = increment % 12;
				increment -= nYears * 12;
			}
			else
			{
				nYears = -(increment % 12);
				increment -= nYears * 12;
			}
			year += nYears;
		}
		month += increment;
		if (month > 12)
		{
			month -= 12;
			year++;
		}
		else if (month < 0)
		{
			month += 12;
			year--;
		}
/*
**		Adjust day, if necessary
*/
		daysInMonth[1] = leap(year) ? 29 : 28;	/* adjust February for leap year */
		if (day > daysInMonth[month - 1])
			day = daysInMonth[month - 1];
		date = [NSString  stringWithFormat:@"%4d%02d%02d",year,month,day];
	}
	else
	{
		sscanf([date  cString],"%4d%2d%2d",&year,&month,&day);
		if (leap(year) && month == 2 && day == 29)
		{
			year += increment;
			if (!leap(year))
				day = 28;
			date = [NSString  stringWithFormat:@"%4d%02d%02d",year,month,day];
		}
		else
		{
			year += increment;
			date = [NSString  stringWithFormat:@"%4d%@",year,[date  substringFromIndex:4]];
		}
	}
	[returnVariable  setValue:date];
	return (returnVariable);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	instance = self;
	instanceInitialized = YES;
	return self;
}

@end