#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import <SFDynamicString.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFCaseInsensitiveString.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFCaseInsensitiveString : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	[super  initFor:sender];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:TEXT];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Text"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (TEXT);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"CIString function translates a character string of TEXT data type into a string "
		    @"suitable for use as the key in the case-insensitive LIKE / NOT LIKE search "
		    @"based on Sybase Transact-SQL rules. E.g. if the original string is ABC, the "
		    @"translated search key is [aA][bB][cC].\n\nThe function takes the following input "
		    @"parameter:\n<Text> - original search key\n");
}

- (NSString *) functionName
{
	return (@"CIString");
}

- execute: (NSArray *) parmList
{
	NSString *text;
/*
**	Build the case-insensitive search key
*/
	[super  execute:nil];
	text = [[parmList  objectAtIndex:0]  value];
	text = [database  caseInsensitiveStringFor:text];
	[returnVariable  setValue:text];
	return (returnVariable);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	[parameterList  release];
	[returnVariable  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end