#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import <SFStorage.h>
#import <SFDynamicString.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFCopyRecord.h"
#import "SFGlobalTemplateManager.h"
#import "SFDataWrapper.h"
#import "SFTesterController.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFCopyRecord : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	[super  initFor:sender];
	columnString = [[SFDynamicString  alloc]  init];
	formatString = [[SFDynamicString  alloc]  init];
	globalTemplateManager = [[SFGlobalTemplateManager  alloc]  init];
	linkStorage = [[SFStorage  alloc]  initCount:0
							  elementSize:sizeof(int)
							  description:""];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:INTEGER];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Resource"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Record Id"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (INTEGER);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"CopyRecord function creates a duplicate copy of the record in the given Resource; "
		    @"the original record is identified by its Record Id. After copying, the Record Id "
		    @"of the newly created record is returned to the caller."
		    @"\n\nThe function takes the following input parameters:\n"
		    @"<Resource> - Name of Subledger or Global table\n"
		    @"<Record Id> - Unique integer identifier of the record to be duplicated\n");
}

- (NSString *) functionName
{
	return (@"CopyRecord");
}

- execute: (NSArray *) parmList
{
	int i, count, idNumber, dataType, retCode, rowId;
	short linkId;
	id container, list;
	SFDataWrapper *rowIdWrapper;
	char type;
	NSString *master, *fieldName, *format, *table, *fieldList, *indexName, *searchKey;
	BOOL isTesting;
/*
**	Build the list of fields and formats
*/
	[super  execute:nil];
	isTesting = [[delegate  delegate]  isKindOfClass:[SFTesterController  class]];
	master = [[parmList  objectAtIndex:0]  value];
	count = [globalTemplateManager  fieldCountFor:master];
	[columnString  copyString:@"SF_ID,"];
	[formatString  copyString:@"INT,"];
	[linkStorage  empty];
	for (i = 0; i < count; i++)
	{
		idNumber = [globalTemplateManager  fieldIdAt:i in:master];
		if (idNumber <= 0)
			continue;
		linkId = [globalTemplateManager  fieldLinkAt:i in:master];
		if (linkId > 0)
		{
			if ([self  addLinkIfAbsent:linkId] == NO)
				continue;
			fieldName = [NSString  stringWithFormat:@"SF_ID%d",linkId];
			format = @"INT";
		}
		else
		{
			fieldName = [globalTemplateManager  fieldSystemNameAt:i in:master];
			dataType = [globalTemplateManager  fieldDataTypeAt:i in:master];
			format = [SFDataBase  dataFormatFor:dataType];
		}
		[columnString  concatenate:fieldName];
		[formatString  concatenate:format];
		[columnString  concatenate:@","];
		[formatString  concatenate:@","];
	}
	[columnString  concatenate:@"SF_UserId,SF_DateChanged,"
						       @"SF_TimeChanged,SF_TimeStamp"];
	[formatString  concatenate:@"STRING,STRING,STRING,SMALL"];
	type = [globalTemplateManager  typeOfTemplate:master];
	if (type == 'S')
	{
/*
**		Add fields specific to the subledger master
*/
		[columnString  concatenate:@",SF_IDG,SF_MatchFieldId,SF_MatchStrategy"];
		[formatString  concatenate:@",INT,SMALL,CHAR"];
	}
/*
**	Retrieve requested record
*/
	rowId = -1;
	[returnVariable  setValue:&rowId];
	rowId = *((int *) [[parmList  objectAtIndex:1]  value]);
	searchKey = [NSString  stringWithFormat:@"where SF_ID = %d",rowId];
	table = [NSString  stringWithFormat:@"SF_MT%@",master];
	retCode = [database  getRowsFor:[columnString  string]
					    fromTable:table
					    conditions:searchKey
					    convertAs:[formatString  string]
					    intoContainer:&container];
	if (retCode == FAIL)
		return (returnVariable);
	count = [container  count];
	if (count != 1)
		return (returnVariable);
/*
**	Get next available Record Id number
*/
	list = [container  objectAtIndex:0];
	rowIdWrapper = [list  objectAtIndex:0];
	[database  getRowsFor:@"max(SF_ID)"
			  fromTable:table
			  conditions:nil
			  convertAs:@"INT"
			  intoVars:&rowId];
	while ([database  getNextRow] == SUCCEED);
		rowId++;
	[rowIdWrapper  setData:&rowId];
	[database  beginTransaction];
	[database  setDisplayErrorMessageEnabled:YES];
	retCode = [database  addRowFor:[columnString  string]
					    inTable:table
					    convertAs:[formatString  string]
					    fromContainer:list];
	[database  setDisplayErrorMessageEnabled:NO];
	if (retCode == FAIL)
	{
		[database  rollbackTransaction];
		if ([database  errorNumber] == DUPLICATE_KEY)
		{
			indexName = [database  nameOfObjectingIndex];
			fieldList = [globalTemplateManager  fieldListOfIndex:indexName in:master];
			NSRunAlertPanel(@"Duplicate data", @"The following field(s) contain values "
						        @"already found in %@:\n%@", @"OK", nil, nil, master,fieldList);
		}
		rowId = -2;
		[returnVariable  setValue:&rowId];
		return (returnVariable);
	}
	if (isTesting)
		[database  rollbackTransaction];
	else
		[database  commitTransaction];
	[returnVariable  setValue:&rowId];
	return (returnVariable);
}

- (BOOL) addLinkIfAbsent: (int) number
{
	int i, count, linkId;
	count = [linkStorage  count];
	for (i = 0; i < count; i++)
	{
		linkId = *((int *) [linkStorage  elementAt:i]);
		if (linkId == number)
			return (NO);
	}
	[linkStorage  addElement:&number];
	return (YES);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[columnString  release];
	[formatString  release];
	[linkStorage  release];
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
	[parameterList  release];
	[returnVariable  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end