#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFCurrentSystemStateFunction.h"

static id instance = nil;					/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFCurrentSystemStateFunction : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	if (instanceInitialized)
		return (instance);
	[super  initFor:sender];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:INTEGER];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (INTEGER);
}

- (NSArray *) parameterList
{
	return nil;
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"CurrentSystemState function returns integer number of the system state that existed "
		    @"at the moment of execution of this function."
		    @"\n\nThe function takes no input parameters");
}

- (NSString *) functionName
{
	return (@"CurrentSystemState");
}

- execute: (NSArray *) parmList
{
	NSString *date, *time, *searchKey;
	short state;
	int systemState;
	[super  execute:nil];
	[database  getCurrentDate:&date time:&time];
/*
**	Execute function
*/
	searchKey = [NSString  stringWithFormat:@"where SF_DateChanged + SF_TimeChanged <= "
							@"'%@%@'",date,time];
	[database  getRowsFor:@"max(State)"					  fromTable:SYSTEMSTATE
			  conditions:searchKey
			  convertAs:@"SMALL"
			  intoVars:&state];
	while ([database  getNextRow] == SUCCEED);
	systemState = (int) state;
	[returnVariable  setValue:&systemState];
	return (returnVariable);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	instance = self;
	instanceInitialized = YES;
	return self;
}

@end