#import <libtools.h>
#import <DefinedConstants.h>
#import <SFFiscalPeriodManager.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFDateToPeriod.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFDateToPeriod : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	self = [super  initFor:sender];
	fiscalPeriodManager = [[SFFiscalPeriodManager  alloc]  init];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:ACCPERIOD];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Date"
						dataType:DATE
						displayedDataType:DATE
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (ACCPERIOD);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"DateToPeriod function returns the fiscal period of the given date. "
			@"The date is passed to the function in YYYYMMDD format, and the "
			@"fiscal period is returned in YY/PPP format where YY is the fiscal "
			@"year, and PPP is the number of period with leading zeros\n\n"
			@"The function takes the following input parameter:\n"
			@"<Date> - date in YYYYMMDD format\n");
}

- (NSString *) functionName
{
	return (@"DateToPeriod");
}

- execute: (NSArray *) parmList
{
	NSString *date;
	SFPeriodData *periodData;
	[super  execute:nil];
	date = [[parmList  objectAtIndex:0]  asciiValue];
	fiscalPeriodManager = [SFFiscalPeriodManager  instance];
	if (fiscalPeriodManager == nil)
		fiscalPeriodManager = [[SFFiscalPeriodManager  alloc]  init];
	periodData = [fiscalPeriodManager  periodForDate:date];
	[returnVariable  setValue:[periodData  periodStr]];
	return (returnVariable);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
	[parameterList  release];
	[self  initFor:delegate];
	instance = self;
	instanceInitialized = YES;
	return self;
}

@end