#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFDebitCreditFunction.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFDebitCreditFunction : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	[super  initFor:sender];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Amount"
						dataType:MONEY
						displayedDataType:MONEY
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Debit"
						dataType:MONEY
						displayedDataType:MONEY
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:NO];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Credit"
						dataType:MONEY
						displayedDataType:MONEY
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:NO];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (UNKNOWN);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"DebitOrCredit function takes the original Amount and places it into Debit "
		    @"or Credit parameters, so that for positive Amount, Debit = Amount and "
		    @"Credit = 0, and for negative Amount Credit = Amount and Debit = 0."
		    @"\n\nThe function takes the following input parameters:\n"
		    @"<Amount> - original amount, positive or negative\n"
		    @"<Debit> - debit amount, contains Amount if it is positive, otherwise 0\n"
		    @"<Credit> - credit amount, contains Amount if it is negative, otherwise 0");
}

- (NSString *) functionName
{
	return (@"DebitOrCredit");
}

- execute: (NSArray *) parmList
{
	SFVariable *variable;
	double amount, debit, credit, *value;
	[super  execute:nil];
	value = ((double *) [[parmList  objectAtIndex:0]  value]);
	if (value == NULL)
		amount = 0.0;
	else
		amount = *value;
	if (amount >= 0.0)
	{
		debit = amount;
		credit = 0.0;
	}
	else
	{
		debit = 0.0;
		credit = amount;
	}
	variable = [parmList  objectAtIndex:1];
	[variable  setValue:&debit];
	[variable  setPrecision:2];
	variable = [parmList  objectAtIndex:2];
	[variable  setValue:&credit];
	[variable  setPrecision:2];
	return nil;
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	instance = self;
	instanceInitialized = YES;
	return self;
}

@end