#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import <SFDynamicString.h>
#import "SFVariable.h"
#import "SFDataWrapper.h"
#import "SFSystemProcedure.h"
#import "SFExtraFieldFunction.h"
#import "SFGraphicFunctionStatementBox.h"
#import "SFExtraFieldManager.h"
#import "SFParameterData.h"
#import "SFFiscalPeriod.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

static NSString *descriptionText = @"ExtraField function returns the contents of the Extra field "
						 @"from the transaction record. The function should be used in "
						 @"conjunction with the retrieval from the Transaction resource.\n\n"
						 @"The function takes the following input parameters:\n"
						 @"<Extra table> - name of the Extra template that is returned by "
						 @"the Transaction resource\n"
						 @"<Record Id> - unique identifier of the Transaction record that is "
						 @"returned by the Transaction resource\n"
						 @"<Field name> - system name of the field to be retrieved\n";

@implementation SFExtraFieldFunction : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	int returnType;
	NSString *name;
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	[super  initFor:sender];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	searchKey = [[SFDynamicString  alloc]  init];
	extraFieldManager = [[SFExtraFieldManager  alloc]  init];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Extra table"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Record Id"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Extra field"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:NO
						isSwitchDisplayed:NO];
	[parameter  setStringTable:[extraFieldManager  fieldNameList]];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	name = [[parameter  stringTable]  objectAtIndexIfExists:0];
	returnType = [extraFieldManager  dataTypeOfField:name];
	[returnVariable  setDataType:returnType];
	instanceInitialized = YES;
	return self;
}

- (int) returnTypeFor: (id) graphic
{
	int returnType;
	NSString *fieldName;
	BOOL isRunTime;
	[graphic  getParameter:&fieldName runTime:&isRunTime at:2];
	extraFieldManager = [SFExtraFieldManager  instance];
	if (extraFieldManager == nil)
		extraFieldManager = [[SFExtraFieldManager alloc]  init];
	returnType = [extraFieldManager  dataTypeOfField:fieldName];
	return (returnType);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (descriptionText);
}

- (NSString *) functionName
{
	return (@"ExtraField");
}

- execute: (NSArray *) parmList
{
	id container, list;
	SFParameterData *parameter;
	NSString *tableName, *templateName, *fieldName, *format;
	int recordId, dataType, length, retCode;
	SFDataWrapper *dataWrapper;
	[super  execute:nil];
	extraFieldManager = [SFExtraFieldManager  instance];
	if (extraFieldManager == nil)
	{
		extraFieldManager = [[SFExtraFieldManager  alloc]  init];
		parameter = [parameterList  objectAtIndex:2];
		[parameter  setStringTable:[extraFieldManager  fieldNameList]];
		fieldName = [[parameter  stringTable]  objectAtIndex:0];
		dataType = [extraFieldManager  dataTypeOfField:fieldName];
		[returnVariable  setDataType:dataType];
	}
	templateName = [[parmList  objectAtIndex:0]  asciiValue];
	recordId = (*((int *) [[parmList  objectAtIndex:1]  value]));
	fieldName = [[parmList  objectAtIndex:2]  asciiValue];
	if ([extraFieldManager  isField:fieldName inTemplate:templateName] == NO)
		[returnVariable  setValue:NULL];
	else
	{
		tableName = [NSString  stringWithFormat:@"SF_ET%@",templateName];
		[searchKey  sprintf:@"where SF_ID = %d",recordId];
		dataType = [extraFieldManager  dataTypeOfField:fieldName];
		switch (dataType)
		{
			case INTEGER:
				format = @"INT";
				break;
			case SHORT:
				format = @"SMALL";
				break;
			case TINY:
				format = @"TINY";
				break;
			case LOGICAL:
				format = @"LOGICAL";
				break;
			case FLOAT:
				format = @"FLT8";
				break;
			case MONEY:
				format = @"MONEY";
				break;
			case NOTE:
			case IMAGE:
				format = @"BLOB";
				break;
			default:
			case DATE:
				format = @"STRING";
				break;
			case TIME:
				format = @"STRING";
				break;
			case TEXT:
			case GL:
			case GLPREFIX:
			case GLNUMBER:
			case SLNUMBER:
			case ACCPERIOD:
			case GLACCLIST:
			case GLACCGROUP:
				length = [extraFieldManager  lengthOfField:fieldName];
				if (length == 1)
					format = @"CHAR";
				else
					format = @"STRING";
				break;
		}
		[returnVariable  setDataType:[self  returnType]];
		retCode = [database  getRowsFor:fieldName
						   fromTable:tableName
						   conditions:[searchKey  string]
						   convertAs:format
						   intoContainer:&container];
		if (retCode == SUCCEED)
		{
			if ([container  count] > 0)
			{
				list = [container  objectAtIndex:0];
				dataWrapper = [list  objectAtIndex:0];
				[returnVariable  setValueFrom:dataWrapper];
			}
			else
				[returnVariable  setValue:NULL];
		}
		else
			[returnVariable  setValue:NULL];
	}
	return (returnVariable);
}

- (void) dealloc
{
	[searchKey  release];
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	[parameterList  release];
	[returnVariable  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end