#import <libtools.h>
#import <DefinedConstants.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFIntegerToText.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFIntegerToText : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	[super  initFor:sender];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:TEXT];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Number"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (TEXT);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"IntegerToText function translates an integer number into a character string of TEXT "
		    @"data type.\n\nThe function takes the following input parameter:\n"
		    @"<Number> - number to convert to text\n");
}

- (NSString *) functionName
{
	return (@"IntegerToText");
}

- execute: (NSArray *) parmList
{
	int number;
	NSString *text;
/*
**	Build the list of fields and formats
*/
	[super  execute:nil];
	number = [[parmList  objectAtIndex:0]  intValue];
	text = [NSString  stringWithFormat:@"%d",number];
	[returnVariable  setValue:text];
	return (returnVariable);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	[parameterList  release];
	[returnVariable  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end