#import <libtools.h>
#import <DefinedConstants.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFMoneyToText.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFMoneyToText : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	[super  initFor:sender];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:TEXT];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Number"
						dataType:MONEY
						displayedDataType:MONEY
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Decimal places"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (TEXT);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"MoneyToText function translates a money value into a character string of TEXT "
		    @"data type.\n\nThe function takes the following input parameters:\n"
		    @"<Number> - number to convert to text\n"
		    @"<Decimal places> - number to decimal places to keep in the text string (-1 "
		    @"to inherit it from the input parameter)\n");
}

- (NSString *) functionName
{
	return (@"MoneyToText");
}

- execute: (NSArray *) parmList
{
	double number;
	int precision;
	NSString *text, *format;
	SFVariable *parameter;
/*
**	Build the list of fields and formats
*/
	[super  execute:nil];
	parameter = [parmList  objectAtIndex:0];
	number = [parameter  doubleValue];
	precision = [[parmList  objectAtIndex:1]  intValue];
	if (precision < 0)
		precision = [parameter  precision];
	format = [NSString  stringWithFormat:@"%%-30.%df",precision];
	text = [NSString  stringWithFormat:format,number];
	[returnVariable  setValue:text];
	return (returnVariable);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	[parameterList  release];
	[returnVariable  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end