#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDateFormatManager.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFParameterData.h"
#import "SFPeriodComponentsFunction.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

static NSString *descriptionText = @"PeriodComponentsFunction function parses the contents of "
						 @"period string that is passed to the function in YY/PPP format where "
						 @"YY is the fiscal year, and PPP is the number of period with leading "
						 @"zeros. For example, period 2 of fiscal year 1994 that has total of 12 "
						 @"periods would be formatted as 94/02. It returns separated "
						 @"components of the period as year and period number. E.g. "
						 @"period 94/02 would be parsed as 1994 and 2.\n\n"
						 @"The function takes the following input parameters:\n"
						 @"<Period> - fiscal period in YY/PPP format\n"
						 @"<Year> - integer year of the period, with century\n"
						 @"<Number> - integer number of the period\n";

@implementation SFPeriodComponentsFunction : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	self = [super  initFor:sender];
	dateFormatManager = [[SFDateFormatManager  alloc]  init];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Period"
						dataType:ACCPERIOD
						displayedDataType:ACCPERIOD
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Year"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:NO];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Number"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:NO];
	[parameterList  addObjectWithoutRetaining:parameter];
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (UNKNOWN);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (descriptionText);
}

- (NSString *) functionName
{
	return (@"PeriodComponents");
}

- execute: (NSArray *) parmList
{
	int year2000Threshold, century;
	char *period, buffer[7];
	int year, number;
	[super  execute:nil];
	period = (char *) [[[parmList  objectAtIndex:0]  asciiValue]  cString];
	year2000Threshold = [dateFormatManager  year2000Threshold];
	year = atoi(period);
	if (year > year2000Threshold)
		century = 1900;
	else
		century = 2000;
	 year += century;
	strcpy(buffer,period);
	period = strchr(buffer,'/') + 1;
	number = atoi(period);
	[[parmList  objectAtIndex:1]  setValue:&year];
	[[parmList  objectAtIndex:2]  setValue:&number];
	return nil;
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
	[parameterList  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end