#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import <SFDynamicString.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFPeriodRangeFunction.h"
#import "SFDataWrapper.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

static NSString *descriptionText = @"PeriodRange function calculates starting and ending "
							@"accounting  period that belong to the given Match "
							@"group.\n\nThe function takes the following input parameters:\n"
							@"<GL Prefix> - GL account prefix\n"
							@"<GL Account> - GL account number\n"
							@"<Subledger Id> - internal subledger number, as recorded in "
							@"the subledger and transaction tables\n"
							@"<OPI Match Value> - identifier of the Match group\n\n"
							@"The function returns the starting and ending accounting "
							@"periods of  the given Match group";

@implementation SFPeriodRangeFunction : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	self = [super  initFor:sender];
	searchKey = [[SFDynamicString  alloc]  init];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"GL Prefix"
						dataType:UNKNOWN
						displayedDataType:GLPREFIX
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"GL Account"
						dataType:UNKNOWN
						displayedDataType:GLNUMBER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Subledger Id"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"OPI Match Value"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Starting period"
						dataType:ACCPERIOD
						displayedDataType:ACCPERIOD
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Ending period"
						dataType:ACCPERIOD
						displayedDataType:ACCPERIOD
						ioFlag:OUTPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	instanceInitialized = YES;
	return self;
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (descriptionText);
}

- (NSString *) functionName
{
	return (@"PeriodRange");
}

- execute: (NSArray *) parmList
{
	NSString *account, *prefix, *subledger, *matchValue;
	id container, list, parameter;
	SFDataWrapper *dataWrapper;
	[super  execute:nil];
	prefix = [[parmList  objectAtIndex:0]  asciiValue];
	account = [[parmList  objectAtIndex:1]  asciiValue];
	subledger = [[parmList  objectAtIndex:2]  asciiValue];
	matchValue = [[parmList  objectAtIndex:3]  asciiValue];
/*
**	Set search criteria
*/
	if (isPrefixUsed)
		[searchKey  sprintf:@"where SF_GLPrefix = '%@' and SF_GLNumber = '%@' and "
						@"SF_IDS = %@ and SF_MatchValue = '%@' order by SF_Period",
						prefix,account,subledger,matchValue];
	else
		[searchKey  sprintf:@"where SF_GLNumber = '%@' and SF_IDS = %@ and "
						@"SF_MatchValue = '%@' order by SF_Period",
						account,subledger,matchValue];
	[database  getRowsFor:@"distinct SF_Period"
			  fromTable:OPENITEM
			  conditions:[searchKey  string]
			  convertAs:@"STRING"
			  intoContainer:&container];
	list = [container  objectAtIndex:0];
	if (list != nil)
	{
		dataWrapper = [list  objectAtIndex:0];
		parameter = [parmList  objectAtIndex:4];
		[parameter  setValueFrom:dataWrapper];
		list = [container  lastObject];
		dataWrapper = [list  objectAtIndex:0];
		parameter = [parmList  objectAtIndex:5];
		[parameter  setValueFrom:dataWrapper];
	}
	return nil;
}

- (BOOL) caresForPrefixStatus
{
	return (YES);
}

- (void) dealloc
{
	[searchKey  release];
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	[parameterList  release];
	instanceInitialized = NO;
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end