#import <libtools.h>
#import <DefinedConstants.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFPrefixToText.h"
#import "SFGLAccountManager.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"

static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFPrefixToText : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	self = [super  initFor:sender];
	glAccountManager = [[SFGLAccountManager  alloc]  init];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:TEXT];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"GL Prefix"
						dataType:GLPREFIX
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Segment #"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (TEXT);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"SFPrefixToText function translates a segment of the given prefix into its name "
		    @"as defined by the Account Prototypes, and returns it to the caller."
		    @"\n\nThe function takes the following input parameters:\n"
		    @"<GL Prefix> - GL Prefix\n"
		    @"<Segment #> - Number of the segment in prefix to be translated to text\n");
}

- (NSString *) functionName
{
	return (@"PrefixToText");
}

- execute: (NSArray *) parmList
{
	int retCode, segmentNo;
	id stringTable = nil;
	NSString *prefix, *name;
/*
**	Build the list of fields and formats
*/
	[super  execute:nil];
	prefix = [[parmList  objectAtIndex:0]  value];
	segmentNo = [[parmList  objectAtIndex:1]  intValue];
	glAccountManager = [SFGLAccountManager  instance];
	if (glAccountManager == nil)
		glAccountManager = [[SFGLAccountManager alloc]  init];
	retCode = [glAccountManager  validatePrefix:prefix
							    name:NULL
							    stringTable:&stringTable
							    diagnostics:NULL];
	if (retCode == GV_SUCCESS)
		name = [stringTable  objectAtIndex:segmentNo - 1];
	else
		name = @"";
	[returnVariable  setValue:name];
	return (returnVariable);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	[parameterList  release];
	[returnVariable  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end