#import <libtools.h>
#import <DefinedConstants.h>
#import <SFDatabase.h>
#import <SFDynamicString.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFSubledgerNumberFunction.h"
#import "SFParameterData.h"
#import "NSArrayAdditionalMethods.h"
	
static id instance = nil;						/* Id of the instance of the Class (one only) */
static BOOL instanceInitialized = NO;		/* Flag to track initialization of the instance */

@implementation SFSubledgerNumberFunction : SFSystemFunction

+ alloc
{
	if (instance == nil)
		instance = NSAllocateObject([self  class],0,NULL);
	return (instance);
}

- initFor: (id) sender
{
	SFParameterData *parameter;
	if (instanceInitialized)
		return (instance);
	self = [super  initFor:sender];
	searchKey = [[SFDynamicString  alloc]  init];
	stringKey = [[SFDynamicString  alloc]  init];
	subledgerList = [[NSMutableDictionary  alloc]  initWithCapacity:0];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:TEXT];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"GL Prefix"
						dataType:GLPREFIX
						displayedDataType:GLPREFIX
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"GL Number"
						dataType:GLNUMBER
						displayedDataType:GLNUMBER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Subledger Id"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	instanceInitialized = YES;
	return self;
}

- (int) returnType
{
	return (TEXT);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	return (@"SubledgerNumber function translates internal Subleger Id into the character "
		    @"representation of the Subledger Number."
		    @"\n\nThe function takes the following input parameters:\n"
		    @"<GL Prefix> - GL Prefix of the GL account returned from the Transaction resource\n"
		    @"<GL Number> - GL Number of the GL account returned from the Transaction "
		    @"resource\n"
		    @"<Subledger Id> - Internal identifier of the Subledger Number returned from the "
		    @"Transaction resource");
}

- (NSString *) functionName
{
	return (@"SubledgerNumber");
}

- execute: (NSArray *) parmList
{
	NSString *glPrefix, *glNumber, *slNumber, *master, *tableName, *subledgerString;
	int subledgerId, internalNumber;
	subledgerId = [[parmList  objectAtIndex:2]  intValue];
	if (subledgerId == 0)
	{
		slNumber = @"";
		[returnVariable  setValue:slNumber];
		return (returnVariable);
	}
	[super  execute:nil];
	glPrefix = [[parmList  objectAtIndex:0]  value];
	glNumber = [[parmList  objectAtIndex:1]  value];
	if (isEmptyString(glNumber))
	{
/*
**		Error
*/
		if (NSApp != nil)
			NSRunAlertPanel(@"Run-time error",@"\"GL Number\" parameter in function "
						       @"SubledgerNumber is empty", @"OK", nil, nil);
		else
		{
/*
**			Background job: quit with error message (DataBase "failure" method
**			will send the message text to the main Server and quit the task).
*/
			[database  failure:@"\"GL Number\" parameter in function SubledgerNumber is "
						      @"empty"];
		}
		return nil;
	}
/*
**	Check if parameters have been cached
*/
	[stringKey  sprintf:@"%@%@%d",glPrefix,glNumber,subledgerId];
	subledgerString = [subledgerList  objectForKey:[stringKey  string]];
	if (subledgerString != nil)
	{
		[returnVariable  setValue:subledgerString];
		return (returnVariable);
	}
/*
**	Execute function
*/
	if (isPrefixUsed)
		[searchKey  sprintf:@"where SF_GLPrefix = '%@' and SF_GLNumber = '%@'",
						glPrefix,glNumber];
	else
		[searchKey  sprintf:@"where SF_GLNumber = '%@'",glNumber];
	[database  getRowsFor:@"SF_IDG,SF_Subledger"
			  fromTable:VALIDACCOUNT
			  conditions:[searchKey  string]
			  convertAs:@"INT,STRING"
			  intoVars:&internalNumber,&master];
	while ([database  getNextRow] == SUCCEED);
	if (isEmptyString(master))
	{
		slNumber = @"";
		[returnVariable  setValue:slNumber];
		return (returnVariable);
	}
	[searchKey  sprintf:@"where SF_IDG = %d and SF_ID = %d",internalNumber,subledgerId];
	tableName = [NSString  stringWithFormat:@"SF_MV%@",master];
	[database  getRowsFor:@"SF_SubledgerNumber"
			  fromTable:tableName
			  conditions:[searchKey  string]
			  convertAs:@"STRING"
			  intoVars:&subledgerString];
	while ([database  getNextRow] == SUCCEED);
/*
**	Add new entry to the string table
*/
	[subledgerList  setObject:[stringKey  string]
				 forKey:subledgerString];
	[returnVariable  setValue:subledgerString];
	return (returnVariable);
}

- (BOOL) caresForPrefixStatus
{
	return (YES);
}

- (void) dealloc
{
	instance = nil;
	instanceInitialized = NO;
	[searchKey  release];
	[stringKey  release];
	[subledgerList  release];
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
//	[super  initWithCoder:aDecoder];
	[parameterList  release];
	[self  initFor:delegate];
	instance = self;
	return self;
}

@end