/*
**	Add-on function for STEPforward flowcharts.
**	Release 2.0
**	Copyright (c) 1998, Phoenix Data Trend.  All rights reserved.
**
**	You can use this source code as a prototype to develop your
**	own precompiled functions for inclusion into the STEPforward
**	function inventory.
**
**	Note, however, that Phoenix Data Trend does not provide a
**	warranty or support for the functions developed by the user,
**	nor is Phoenix Data Trend liable for any indirect, special,
**	incidental or consequential damages that the use of such
**	functions may cause.
*/

#import "SFSystemFunction.h"

@interface SFSummaryFunctions : SFSystemFunction
{
	id accounts;					/* Storage of internal GL account numbers */
	id records;					/* Storage of ADDED_RECORD structures */
	id searchKey;					/* DynamicString for SQL WHERE clause */
	id description;					/* DynamicString for function description */
	id fiscalPeriods;				/* Instance of FiscalPeriod class */
	id stringTableManager;			/* StringTableManager to store the string tables */
	id functionNameList;				/* List of function names provided in function */
	int lastGLAccount;				/* Internal number of last used GL account */
	int lastSLAccount;				/* Internal number of last used SL account */
	int functionType;				/* Type of the function (TSI, YTD, CA, DA or OB) */
	NSString *lastListName;			/* Last used List name */
	NSString *lastPrefix;				/* Last used GL prefix */
	NSString *lastEndingDate;		/* Last used ending date */
	NSString *lastSubledger;			/* Last used subledger number string */
	NSString *masterName;			/* Last used Subledger Master name */
	NSString *functionName;			/* Name of the function */
	short lastState;					/* Last used System State */
	BOOL lastSubledgerFlag;		/* Last used SL data flag */
	BOOL lastPostingFlag;			/* Last used posting simulation flag */
	NSCharacterSet *decimalDigitSet;	/* Decimal digit character set */
}

- initFor: (id) sender;
- (void) setFunctionName: (NSString *) name;
- (int) returnType;
- (NSString *) descriptionForFunction: (NSString *) name;
- (NSArray *) functionNameList;
- (short) currentSystemState;
- execute: (NSArray *) parmList;
- (void) getAccountsFromList: (NSString *) listName
	forGlPrefix: (NSString *) prefix;
- (void) extractDataForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledgerNumber: (int) slNumber
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost;
- (void) simulatePostingUpToPeriod: (short) fiscalPeriod
	subledgerNumber: (int) slNumber
	endingDate: (NSString *) endingDate;
- (void) prepareAllDataForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost;
- (void) saveList: (NSString *) listName
	glPrefix: (NSString *) prefix
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost;
- (double) totalSinceInceptionForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost;
- (double) yearToDateForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost;
- (BOOL) caresForPrefixStatus;
- (void) dealloc;
- initWithCoder: (NSCoder *) aDecoder;

@end