#import <libtools.h>
#import <DefinedConstants.h>
#import <SFStorage.h>
#import <SFDatabase.h>
#import <SFDataWrapper.h>
#import <SFDynamicString.h>
#import <SFFiscalPeriodManager.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFSummaryFunctions.h"
#import "SFStringTableManager.h"
#import "SFParameterData.h"
#import "SFPeriodData.h"
#import "SFFiscalPeriod.h"
#import "NSStringAdditionalMethods.h"
#import "NSArrayAdditionalMethods.h"

/*
**	Function subtypes
*/
#define  TSI				    0		/* Total since inception */
#define  YTD				    1		/* Year to date */

static NSString *descriptionText = @"\n\nThe function takes the following input parameters:\n"
						@"<GL Prefix> - specific number or a search key that uses "
						@"standard SQL keywords(\"like\", \"not like\", \"between\" etc.)\n"
						@"<GL Account> - specific number or Account List name\n"
						@"<SL Account> - subledger account number, or blank\n"
						@"<Document Type> - can be \"Actual\" or \"Budget\"\n"
						@"<Document Class> - can be \"Regualr\" or \"Restatement\"\n"
						@"<Ending Date> - ending date of the time interval for summation\n"
						@"<System State> - number of the system state to be \"visible\" to the "
						@"function\n"
						@"<Simulate Posting> = \"Yes\" to simulate posting of all unposted "
						@"documents, \"No\" to honor only actually posted documents";
/*
**	Record ID structure
*/
typedef struct
{
	int accountNumber;		/* Internal GL account number */
	int subledgerNumber;		/* Internal SL account number */
	double totalSinceInception;	/* Total Since Inception amount */
	double yearToDate;		/* Year To Date total amount */
	char docType;			/* Document type */
	char restatement;			/* Document class */
}	SUMMARY_RECORD;

@implementation SFSummaryFunctions : SFSystemFunction

- initFor: (id) sender
{
	SFParameterData *parameter;
	id dataWrapper;
	self = [super  initFor:sender];
	decimalDigitSet = [[NSCharacterSet  decimalDigitCharacterSet]  retain];
	returnVariable = [[SFVariable  alloc]  initName:@"" type:DATAFIELD];
	[returnVariable  setDataType:MONEY];
	[returnVariable  setPrecision:2];
	records = [[SFStorage  alloc]  initCount:0
							elementSize:sizeof(SUMMARY_RECORD)
							description:""];
	accounts = [[SFStorage  alloc]  initCount:0
							elementSize:sizeof(int)
							description:""];
	searchKey = [[SFDynamicString  alloc]  init];
	description = [[SFDynamicString  alloc]  init];
	stringTableManager = [[SFStringTableManager  alloc]  init];
	functionNameList = [[NSMutableArray  alloc]  initWithCapacity:2];
	dataWrapper = [[SFDataWrapper  alloc]  initWithNSString:@"TotalSinceInception"];
	[functionNameList  addObject:dataWrapper];
	dataWrapper = [[SFDataWrapper  alloc]  initWithNSString:@"YearToDate"];
	[functionNameList  addObject:dataWrapper];
/*
**	Create Storage for parameter data and fill it in
*/
	parameterList = [[NSMutableArray  alloc]  initWithCapacity:0];
	parameter = [[SFParameterData  alloc]
						initWithName:@"GL Prefix"
						dataType:UNKNOWN
						displayedDataType:GLPREFIX
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"GL Account"
						dataType:UNKNOWN
						displayedDataType:GLNUMBER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"SL Account"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Document Type"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameter  setStringTable:[stringTableManager  typeList]];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Document Class"
						dataType:TEXT
						displayedDataType:TEXT
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameter  setStringTable:[stringTableManager  restatementList]];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Ending date"
						dataType:DATE
						displayedDataType:DATE
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"System State"
						dataType:INTEGER
						displayedDataType:INTEGER
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  addObjectWithoutRetaining:parameter];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Simulate Posting"
						dataType:LOGICAL
						displayedDataType:LOGICAL
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameter  setStringTable:[stringTableManager  simulatePostList]];
	[parameterList  addObjectWithoutRetaining:parameter];
/*
**	Initialize rest of variables
*/
	lastListName = nil;
	lastPrefix = nil;
	lastSubledger = nil;
	lastEndingDate = nil;
	functionName = nil;
	lastState = 0;
	lastSubledgerFlag = NO;
	lastPostingFlag = NO;
	lastGLAccount = -1;
	masterName = nil;
	fiscalPeriods = nil;
	return self;
}

- (void) setFunctionName: (NSString *) name
{
	functionName = [name  preserve];
	if (stringsAreEqual(name,@"TotalSinceInception"))
		functionType = TSI;
	else if (stringsAreEqual(name,@"YearToDate"))
		functionType = YTD;
}

- (int) returnType
{
	return (MONEY);
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	stringsAreEqual(@"A",@"B");
	if (stringsAreEqual(name,@"TotalSinceInception"))
		[description  copyString:@"TotalSinceInception function calculates total Amount "
							@"for the specified accounts (see parameters below) "
							@"accumulated since inception of the system."];
	else if (stringsAreEqual(name,@"YearToDate"))
		[description  copyString:@"YearToDate function calculates total Amount "
							@"for the specified accounts (see parameters below) "
							@"accumulated since the beginning of the fiscal year "
							@"to the ending date."];
	else
		[description  copyString:@""];
	[description  concatenate:descriptionText];
	return [description  string];
}

- (NSArray *) functionNameList
{
	return (functionNameList);
}

- (short) currentSystemState
{
	NSString *date, *time;
	short state = 1;
/*
**	Get current state from the current date/time
*/
	[database  getCurrentDate:&date time:&time];
	[searchKey  sprintf:@"where SF_DateChanged + SF_TimeChanged <= '%@%@'",date,time];
	[database  getRowsFor:@"max(State)"					  fromTable:SYSTEMSTATE
			  conditions:[searchKey  string]
			  convertAs:@"SMALL"
			  intoVars:&state];
	while ([database  getNextRow] == SUCCEED);
	return (state);
}

- execute: (NSArray *) parmList
{
	NSString *listName, *prefix, *subledger, *docType, *docClass, *endingDate, *posting;	char class, type;
	short state;
	double amount;
	void *value;
	BOOL sPost;
	[super  execute:nil];
	if (parmList == nil)
		return nil;
	fiscalPeriods = [[SFFiscalPeriodManager  alloc]  init];
	prefix = [[parmList  objectAtIndex:0]  asciiValue];
	listName = [[parmList  objectAtIndex:1]  asciiValue];
	subledger = [[parmList  objectAtIndex:2]  asciiValue];
	docType = [[parmList  objectAtIndex:3]  asciiValue];
	docClass = [[parmList  objectAtIndex:4]  asciiValue];
	endingDate = [[parmList  objectAtIndex:5]  asciiValue];
	value = [[parmList  objectAtIndex:6]  value];
	if (value != NULL)
		state = (short) [[parmList  objectAtIndex:6]  intValue];
	else
		state = [self  currentSystemState];
	posting = [[parmList  objectAtIndex:7]  asciiValue];
/*
**	Set Document Class to required format
*/
	if (stringsAreEqual(docClass,@"Regular"))
		class = ' ';
	else if (stringsAreEqual(docClass,@"Restatement"))
		class = 'R';
	else
	{
/*
**		Error
*/
		if (NSApp != nil)
			NSRunAlertPanel(@"Run-time error", @"Parameter \"Document Class\" in function "
						       @"\"%@\" has been given an invalid value (\"%@\"). Valid "
						       @"classes are: \"Regular\" or \"Restatement\".",
						       @"OK", nil, nil, functionName,docClass);
		else
		{
/*
**			Background job: quit with error message (DataBase "failure" method
**			will send the message text to the main Server and quit the task).
*/
			NSString *message;
			message = [NSString  stringWithFormat:@"Parameter \"Document Class\" in "
							@"function \"%@\" has been given an invalid value (\"%@\"). "
							@"Valid classes are: \"Regular\" or \"Restatement\".",
							functionName,docClass];
			[database  failure:message];
		}
		return nil;
	}
/*
**	Set Document Type to required format
*/
	if (stringsAreEqual(docType,@"Actual"))
		type = 'A';
	else if (stringsAreEqual(docType,@"Budget"))
		type = 'B';
	else
	{
/*
**		Error
*/
		if (NSApp != nil)
			NSRunAlertPanel(@"Run-time error", @"Parameter \"Document Type\" in function "
						       @"\"%@\" has been given an invalid value (\"%@\"). Valid types "
						       @"are: \"Actual\" or \"Budget\".",
						       @"OK", nil, nil, functionName,docType);
		else
		{
/*
**			Background job: quit with error message (DataBase "failure" method
**			will send the message text to the main Server and quit the task).
*/
			NSString *message;
			message = [NSString  stringWithFormat:@"Parameter \"Document Type\" in "
							@"function \"%@\" has been given an invalid value (\"%@\"). "
							@"Valid types are: \"Actual\" or \"Budget\".",
							functionName,docType];
			[database  failure:message];
		}
		return nil;
	}
/*
**	Set Posting status to required format
*/
	if ([posting  caseInsensitiveCompare:@"Yes"] == NSOrderedSame)
		sPost = YES;
	else if ([posting  caseInsensitiveCompare:@"No"] == NSOrderedSame)
		sPost = NO;
	else
	{
/*
**		Error
*/
		if (NSApp != nil)
			NSRunAlertPanel(@"Run-time error", @"Parameter \"Simulate Posting\" in function "
						       @"\"%@\" has been given an invalid value (\"%@\"). Valid "
						       @"values are: \"Yes\" or \"No\".",
						       @"OK", nil, nil, functionName,posting);
		else
		{
/*
**			Background job: quit with error message (DataBase "failure" method
**			will send the message text to the main Server and quit the task).
*/
			NSString *message;
			message = [NSString  stringWithFormat:@"Parameter \"Simulate Posting\" in "
								@"function \"%@\" has been given an invalid value "
								@"(\"%@\"). Valid values are: \"Yes\" or \"No\".",
								functionName,posting];
			[database  failure:message];
		}
		return nil;
	}
/*
**	Execute appropriate function
*/
	switch (functionType)
	{
		case TSI:
			amount = [self  totalSinceInceptionForList:listName
						glPrefix:prefix
						subledger:subledger
						documentType:type
						documentClass:class
						endingDate:endingDate
						systemState:state
						simulatePosting:sPost];
			break;
		case YTD:
			amount = [self  yearToDateForList:listName
						glPrefix:prefix
						subledger:subledger
						documentType:type
						documentClass:class
						endingDate:endingDate
						systemState:state
						simulatePosting:sPost];
			break;
	}
	[returnVariable  setValue:&amount];
	return (returnVariable);
}

- (void) getAccountsFromList: (NSString *) listName
	forGlPrefix: (NSString *) prefix
{
/*
**	Get the array of the internal account numbers for the given prefix.
**	If the "listName" is the valid name of the account range list, limit
**	the search to the account ranges in the list. If the "listName" starts
**	with a digit (not a valid list name), treat it as the GL Account and
**	simply get its internal number.
*/ 
	int accountNumber;
	unichar c;
	NSString *table;
	[accounts  empty];
/*
**	Create SQL sequence for search by GL prefix
*/
	[searchKey  copyString:@"where "];
	if (isPrefixUsed && prefix != nil)
	{
		if ([prefix  isInteger])
		{
/*
**			Prefix number
*/
			[searchKey  concatenate:@"SF_GLPrefix = '"];
			[searchKey  concatenate:prefix];
			[searchKey  concatenate:@"'"];
		}
		else
		{
/*
**			Search key string
*/
			[searchKey  concatenate:@"SF_GLPrefix "];
			[searchKey  concatenate:prefix];
		}
		[searchKey  concatenate:@" and "];
	}
	if (isEmptyString(listName))
		return;
	c = [listName  characterAtIndex:0];
	if ([decimalDigitSet  characterIsMember:c])
	{
/*
**		Single account
*/
		[searchKey  concatenate:@"SF_GLNumber ='"];
		[searchKey  concatenate:listName];
		[searchKey  concatenate:@"'"];
		[database  getRowsFor:@"SF_IDG"
				  fromTable:VALIDACCOUNT
				  conditions:[searchKey  string]
				  convertAs:@"INT"
				  intoVars:&accountNumber];
		while ([database  getNextRow] == SUCCEED)
		{
/*
**			Add data to the Storage
*/
			[accounts  addElement:&accountNumber];
		}
	}
	else
	{
/*
**		List of account ranges
*/
		table = [NSString  stringWithFormat:@"%@ SFAlias1, %@ SFAlias2",
			VALIDACCOUNT,ACCTRANGES];
		[searchKey  concatenate:@"SFAlias1.SF_GLNumber between SFAlias2.FromAccount "
							  @"and SFAlias2.ToAccount and SFAlias2.Name = '"];
		[searchKey  concatenate:listName];
		[searchKey  concatenate:@"' order by SFAlias1.SF_GLNumber"];
		[database  getRowsFor:@"distinct SFAlias1.SF_IDG"
				  fromTable:table
				  conditions:[searchKey  string]
				  convertAs:@"INT"
				  intoVars:&accountNumber];
		while ([database  getNextRow] == SUCCEED)
		{
/*
**			Add data to the Storage
*/
			[accounts  addElement:&accountNumber];
		}
	}
}

- (void) extractDataForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledgerNumber: (int) slNumber
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
/*
**	Get totals from TXSUMMARY table for the specified list of GL accounts
**	(or a single account, passed in place of "listName"). The table is
**	searched for the period the ending date belongs to; if the ending date
**	does not fall on the last day of the period, the amounts are adjusted from
**	the Transaction Detail table.
*/
	id periodList;
	int i, count, j, recordsCount, accountNumber, subledgerNumber, fiscalYear, correction;
	short period, systemState, fiscalPeriod;
	double totalSinceInception, yearToDate, amount;
	NSString *buffer, *table, *operator;
	char docType, restatement;
	BOOL fullPeriod, found, slFlag;
	SFPeriodData *periodData;
	SFFiscalPeriod *fiscalData;
	SUMMARY_RECORD record, *recordPtr;
/*
**	If nothing has changed since the last request, skip everything
*/
	slFlag = (slNumber != 0);
	if (lastListName != nil)
	{
		if (stringsAreEqual(listName,lastListName) &&
			stringsAreEqual(prefix,lastPrefix) &&
			stringsAreEqual(endingDate,lastEndingDate) &&
			state == lastState &&
			slFlag == lastSubledgerFlag &&
			slNumber == lastSLAccount &&
			sPost == lastPostingFlag)
			return;
	}
	lastSubledgerFlag = slFlag;
/*
**	Get the list of accounts
*/
	[records  empty];
	[self  getAccountsFromList:listName forGlPrefix:prefix];
	if ([accounts  count] == 0)
		return;
/*
**	Get the accounting period of the ending date
*/
	periodData = [fiscalPeriods  periodForDate:endingDate];
	if (periodData == nil)
	{
/*
**		Date is outside the fiscal period range. If it is prior to the end
**		of the first fiscal period, quit.
*/
		periodList = [fiscalPeriods  periodList];
		if ([periodList  count] == 0)
			return;
		fiscalData = [periodList  objectAtIndex:0];
		if ([endingDate  compare:[fiscalData  endDate]] == NSOrderedAscending)
			return;
		count = [periodList  count];
		if (count == 0)
			return;
		fiscalData = [periodList  objectAtIndex:count - 1];
		endingDate = [fiscalData  endDate];
		periodData = [fiscalPeriods  periodForDate:endingDate];
	}
	fiscalPeriod = [periodData  internal];
	fullPeriod = stringsAreEqual([periodData  endDate],endingDate);
	if (fullPeriod == NO)
		correction = 1;
	else
		correction = 0;
/*
**	If simulated posting is requested, do it here
*/
	if (sPost)
	{
		[self  simulatePostingUpToPeriod:fiscalPeriod
			subledgerNumber:slNumber
			endingDate:endingDate];
	}
/*
**	Read data for each account, based on the latest 
**	available period (relative to the ending date) and state.
*/
	count = [accounts  count];
	table = [NSString  stringWithFormat:@"%@ A",TXSUMMARY];
	if (slFlag)
	{
		if (count == 1)
		{
			operator = @"=";
			subledgerNumber = slNumber;
		}
		else
		{
			operator = @"!=";
			subledgerNumber = 0;
		}
	}
	else
	{
		operator = @"=";
		subledgerNumber = 0;
	}
	for (i = 0; i < count; i++)
	{
		accountNumber = *((int *) [accounts  elementAt:i]);
		buffer = [NSString  stringWithFormat:@"%d",accountNumber];
		[searchKey  sprintf:@"where SF_IDG = %d and SF_IDS %@ %d and "
						@"SF_SystemState <= %d and SF_AccountingPeriod = "
						@"(select max(SF_AccountingPeriod) from %@ B where "
						@"B.SF_IDG = A.SF_IDG and B.SF_IDS = A.SF_IDS and "
						@"A.SF_SystemState = B.SF_SystemState and "
						@"A.SF_DocType = B.SF_DocType and "
						@"A.SF_Restatement = B.SF_Restatement and "
						@"SF_AccountingPeriod <= %d)",
						accountNumber,operator,subledgerNumber,state,TXSUMMARY,
						fiscalPeriod - correction];
		[database  getRowsFor:@"SF_IDS,SF_TotalSinceInception,SF_AnnualTotal,"
						        @"SF_AccountingPeriod,SF_SystemState,"
						        @"SF_DocType,SF_Restatement"
				  fromTable:table
				  conditions:[searchKey  string]
				  convertAs:@"INT,MONEY,MONEY,SMALL,SMALL,CHAR,CHAR"
				  intoVars:&subledgerNumber,&totalSinceInception,&yearToDate,
				  		 &period,&systemState,&docType,&restatement];
		while ([database  getNextRow] == SUCCEED)
		{
/*
**			Inspect each returned row. If its Accounting Period differs
**			from the requested one, "promote" the period, adjusting
**			the Credit, Debit and Year-to-Date values accordingly.
*/
			if (period != fiscalPeriod)
			{
				fiscalYear = [fiscalPeriods  yearForPeriod:period];
				if (fiscalYear != [periodData  year])
					yearToDate = 0.0;
			}
			systemState = state;
/*
**			Try to update record in Storage (assuming it exists)
*/
			recordsCount = [records  count];
			for (j = 0, found = NO; j < recordsCount; j++)
			{
				recordPtr = (SUMMARY_RECORD *) [records  elementAt:j];
				if (recordPtr->accountNumber == accountNumber &&
					recordPtr->subledgerNumber == subledgerNumber &&
					recordPtr->docType == docType &&
					recordPtr->restatement == restatement)
				{
/*
**					Record exists - update it
*/
					recordPtr->totalSinceInception += totalSinceInception;
					recordPtr->yearToDate += yearToDate;
					found = YES;
					break;
				}
			}
			if (found == NO)
			{
/*
**				Record does not exis - add it
*/
				record.accountNumber = accountNumber;
				record.subledgerNumber = subledgerNumber;
				record.totalSinceInception = totalSinceInception;
				record.yearToDate = yearToDate;
				record.docType = docType;
				record.restatement = restatement;
				[records  addElement:&record];
			}
		}
	}
	if (fullPeriod == NO)
	{
/*
**		Add missing data from TRANSACTION table
*/
		recordPtr = (SUMMARY_RECORD *) [records  elementAt:i];
		if (count == 1 && slNumber != 0)
			[searchKey  sprintf:@"where SF_AccountingPeriod = %d "
							@"and SF_DocumentDate <= '%@' and SF_SystemState <= %d "
							@"and SF_Posted = 'P' and SF_IDS = %d and SF_IDG in (",
							fiscalPeriod,endingDate,state,slNumber];
		else
			[searchKey  sprintf:@"where SF_AccountingPeriod = %d "
							@"and SF_DocumentDate <= '%@' and SF_SystemState <= %d "
							@"and SF_Posted = 'P' and SF_IDG in (",
							fiscalPeriod,endingDate,state];
		count = [accounts  count];
		for (j = 0; j < count; j++)
		{
			accountNumber = *((int *) [accounts  elementAt:j]);
			buffer = [NSString  stringWithFormat:@"%d",accountNumber];
			if (j) [searchKey  concatenate:@","];
			[searchKey  concatenate:buffer];
		}
		[searchKey  concatenate:@")"];
		[database  getRowsFor:@"SF_IDG,SF_IDS,SF_Amount,SF_DocType,SF_Restatement"
				  fromTable:TRANSACTION
				  conditions:[searchKey  string]
				  convertAs:@"INT,INT,MONEY,CHAR,CHAR"
				  intoVars:&accountNumber,&subledgerNumber,&amount,&docType,
				      		 &restatement];
		while ([database  getNextRow] == SUCCEED)
		{
			if (slFlag == NO)
				subledgerNumber = 0;
/*
**			Try to update record (assuming it exists)
*/
			count = [records  count];
			for (j = 0, found = NO; j < count; j++)
			{
				recordPtr = (SUMMARY_RECORD *) [records  elementAt:j];
				if (recordPtr->accountNumber == accountNumber &&
					recordPtr->subledgerNumber == subledgerNumber &&
					recordPtr->docType == docType &&
					recordPtr->restatement == restatement)
				{
/*
**					Record exists - update it
*/
					recordPtr->totalSinceInception += amount;
					recordPtr->yearToDate += amount;
					found = YES;
					break;
				}
			}
			if (found == NO)
			{
/*
**				Record does not exist - add it
*/
				record.accountNumber = accountNumber;
				record.subledgerNumber = subledgerNumber;
				record.totalSinceInception = amount;
				record.yearToDate = amount;
				record.docType = docType;
				record.restatement = restatement;
				[records  addElement:&record];
			}
		}
	}
}

- (void) simulatePostingUpToPeriod: (short) fiscalPeriod
	subledgerNumber: (int) slNumber
	endingDate: (NSString *) endingDate
{
/*
**	Post all completed (but not yet posted) documents from
**	TRANSACTION table into Storage
*/
	int accountNumber, subledgerNumber, fiscalYear, year, i, count;
	short period;
	char docType, docRestatement;
	NSString *buffer;
	double amount, ytdIncrement;
	BOOL found;
	SUMMARY_RECORD record, *recordPtr;
	fiscalYear = [fiscalPeriods  yearForPeriod:fiscalPeriod];
	count = [accounts  count];
	if (count == 1 && slNumber != 0)
		[searchKey  sprintf:@"where (SF_AccountingPeriod < %d or (SF_AccountingPeriod = %d "
						@"and SF_DocumentDate <= '%@')) and SF_Posted = 'C' and "
						@"SF_IDS = %d and SF_IDG in (",fiscalPeriod,fiscalPeriod,
						endingDate,slNumber];
	else
		[searchKey  sprintf:@"where (SF_AccountingPeriod < %d or (SF_AccountingPeriod = %d "
						@"and SF_DocumentDate <= '%@')) and SF_Posted = 'C' and "
						@"SF_IDG in (",fiscalPeriod,fiscalPeriod,endingDate];
/*
**	Set the list of accounts to search
*/
	for (i = 0; i < count; i++)
	{
		accountNumber = *((int *) [accounts  elementAt:i]);
		buffer = [NSString  stringWithFormat:@"%d",accountNumber];
		if (i) [searchKey  concatenate:@","];
		[searchKey  concatenate:buffer];
	}
	[searchKey  concatenate:@")"];
/*
**	Search the table
*/
	[database  getRowsFor:@"SF_IDG,SF_IDS,SF_AccountingPeriod,SF_Amount,SF_DocType,"
					        @"SF_Restatement"
			  fromTable:TRANSACTION
			  conditions:[searchKey  string]
			  convertAs:@"INT,INT,SMALL,MONEY,CHAR,CHAR"
			  intoVars:&accountNumber,&subledgerNumber,&period,&amount,
					 &docType,&docRestatement];
	while ([database  getNextRow] == SUCCEED)
	{
		if (slNumber == 0)
			subledgerNumber = 0;
/*
**		Inspect each returned row. If its Accounting Period differs
**		from the requested one, "promote" the period, adjusting
**		the Credit, Debit and Year-to-Date values accordingly.
*/
		ytdIncrement = amount;
		if (period != fiscalPeriod)
		{
			year = [fiscalPeriods  yearForPeriod:period];
			if (fiscalYear != year)
				ytdIncrement = 0.0;
		}
/*
**		Try to update record (assuming it exists)
*/
		count = [records  count];
		for (i = 0, found = NO; i < count; i++)
		{
			recordPtr = (SUMMARY_RECORD *) [records  elementAt:i];
			if (recordPtr->accountNumber == accountNumber &&
				recordPtr->subledgerNumber == subledgerNumber &&
				recordPtr->docType == docType &&
				recordPtr->restatement == docRestatement)
			{
/*
**				Record exists - update it
*/
				recordPtr->totalSinceInception += amount;
				recordPtr->yearToDate += ytdIncrement;
				found = YES;
				break;
			}
		}
		if (found == NO)
		{
/*
**			Record does not exis - add it
*/
			record.accountNumber = accountNumber;
			record.subledgerNumber = subledgerNumber;
			record.totalSinceInception = amount;
			record.yearToDate = ytdIncrement;
			record.docType = docType;
			record.restatement = docRestatement;
			[records  addElement:&record];
		}
	}	
}

- (void) prepareAllDataForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
	int accountNumber, subledgerAccount;
	NSString *subledgerName;
	unichar c;
	BOOL subledgerFlag, isDigit;
/*
**	Next we will try to find the GL and SL account numbers.
**	This can be done only if the "listName" is a single GL account,
**	and GL prefix does not have any wildcards.
*/
	subledgerFlag = !isEmptyString(subledger);
	if (isEmptyString(listName))
		isDigit = NO;
	else
	{
		c = [listName  characterAtIndex:0];
		isDigit = [decimalDigitSet  characterIsMember:c];
	}
	if (isDigit && [prefix  isInteger])
	{
		accountNumber = lastGLAccount;
		if (!stringsAreEqual(listName,lastListName) || !stringsAreEqual(prefix,lastPrefix))
		{
/*
**			Find the name of the Subledger Master
*/
			subledgerName = nil;
			[masterName  release];
			masterName = nil;
			if (isPrefixUsed)
				[searchKey  sprintf:@"where SF_GLPrefix = '%@' and SF_GLNumber = '%@'",
								prefix,listName];
			else
				[searchKey  sprintf:@"where SF_GLNumber = '%@'",listName];
			[database  getRowsFor:@"SF_IDG,SF_Subledger"
					   fromTable:VALIDACCOUNT
					   conditions:[searchKey  string]
					   convertAs:@"INT,STRING"
					   intoVars:&accountNumber,&subledgerName];
			while ([database  getNextRow] == SUCCEED)
				subledgerName = [subledgerName  truncatedString];
			if (!isEmptyString(subledgerName))
				masterName = [[NSString  alloc]
					initWithFormat:@"SF_MV%@",subledgerName];
		}
/*
**		If the subledger is given, find its internal number (this can be done
**		only if the "listName" is a single GL account, and GL prefix does
**		not have any wildcards. In other words, we can only deal with a
**		concrete Subledger Master, attached to a known GL Number).
*/
		if (subledgerFlag == YES)
		{
			if (!isEmptyString(masterName) && (!stringsAreEqual(subledger,lastSubledger)
				|| accountNumber != lastGLAccount))
			{
				[lastSubledger  release];
				lastSubledger = [subledger  preserve];
/*
**				Find internal number of the given subledger number
*/
				lastSLAccount = 0;
				[searchKey  sprintf:@"where SF_SubledgerNumber = '%@' and SF_IDG = %d",
								subledger,accountNumber];
				[database  getRowsFor:@"SF_ID"
						   fromTable:masterName
						   conditions:[searchKey  string]
						   convertAs:@"INT"
						   intoVars:&subledgerAccount];
				while ([database  getNextRow] == SUCCEED);
			}
			else
				subledgerAccount = lastSLAccount;
		}
		else
			subledgerAccount = 0;
		lastGLAccount = accountNumber;
	}
	else
	{
		lastGLAccount = -1;
		subledgerAccount = 0;
	}
/*
**	Get all summary values for the given list of GL Accounts
**	(and subledger, if it is not NULL), up to the given ending date.
*/
	[self  extractDataForList:listName
		 glPrefix:prefix
		 subledgerNumber:subledgerAccount
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
/*
**	Save parameters for subsequent testing
*/
	lastSLAccount = subledgerAccount;
	[self  saveList:listName
		 glPrefix:prefix
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
}

- (void) saveList: (NSString *) listName
	glPrefix: (NSString *) prefix
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
	[lastListName  release];
	lastListName = [listName  preserve];
	[lastPrefix  release];
	lastPrefix = [prefix  preserve];
	[lastEndingDate  release];
	lastEndingDate = [endingDate  preserve];
	lastState = state;
	lastPostingFlag = sPost;
}

- (double) totalSinceInceptionForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
/*
**	Return Total Since Inception value for the given list of GL Accounts
**	(and subledger, if it is not NULL), up to the given ending date.
*/
	int i, count;
	double total;
	SUMMARY_RECORD *record;
	[self  prepareAllDataForList:listName
		 glPrefix:prefix
		 subledger:subledger
		 documentType:docType
		 documentClass:restatement
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
/*
**	Calculate Total Since Inception for the given GL account
**	(and subledger, if required), observing the required type and
**	class of the documents.
*/
	total = 0.0;
	count = [records  count];
	for (i = 0; i < count; i++)
	{
		record = (SUMMARY_RECORD *) [records  elementAt:i];
		if (lastGLAccount != -1 && record->accountNumber != lastGLAccount)
			continue;
		if (record->subledgerNumber != lastSLAccount)
			continue;
		if (record->docType != docType)
			continue;
		if (restatement != 'B')
		{
			if (record->restatement != restatement)
				continue;
		}
		total += record->totalSinceInception;
	}
	return (total);
}

- (double) yearToDateForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
/*
**	Return Year To Date value for the given list of GL Accounts
**	(and subledger, if it is not NULL), up to the given ending date.
*/
	int i, count, year;
	NSString *date;
	double total;
	NSString *addToDate(NSString *date, int nDays);
	SUMMARY_RECORD *record;
	SFPeriodData *periodData;
	[self  prepareAllDataForList:listName
		 glPrefix:prefix
		 subledger:subledger
		 documentType:docType
		 documentClass:restatement
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
/*
**	Check the date here. If it is the first day of the fiscal year to which
**	the date points, exit with 0.00 as the result. If not, subtract one from the date
**	and proceed with the new date.
*/
	periodData = [fiscalPeriods  periodForDate:endingDate];
	if (periodData != nil)
	{
		year = [periodData  year];
		date =  addToDate(endingDate,-1);
		periodData = [fiscalPeriods  periodForDate:date];
		if (periodData == nil || year != [periodData  year])
			return (0.0);
		endingDate = date;
	}
	else
		return (0.0);
/*
**	Calculate Year To Date for the given GL account
**	(and subledger, if required), observing the required type and
**	class of the documents.
*/
	total = 0.0;
	count = [records  count];
	for (i = 0; i < count; i++)
	{
		record = (SUMMARY_RECORD *) [records  elementAt:i];
		if (lastGLAccount != -1 && record->accountNumber != lastGLAccount)
			continue;
		if (record->subledgerNumber != lastSLAccount)
			continue;
		if (record->docType != docType)
			continue;
		if (restatement != 'B')
		{
			if (record->restatement != restatement)
				continue;
		}
		total += record->yearToDate;
	}
	return (total);
}

- (BOOL) caresForPrefixStatus
{
	return (YES);
}

- (void) dealloc
{
	[decimalDigitSet  release];
	[functionNameList  release];
	[accounts  release];
	[records  release];
	[searchKey  release];
	[description  release];
	[lastListName  release];
	[lastPrefix  release];
	[lastSubledger  release];
	[lastEndingDate  release];
	[functionName  release];
	[masterName  release];
	[super  dealloc];
}

- initWithCoder: (NSCoder *) aDecoder
{
	[functionNameList  release];
	[parameterList  release];
	[returnVariable  release];
	[self  initFor:delegate];
	return self;
}

@end