#import <libtools.h>
#import <DefinedConstants.h>
#import <SFStorage.h>
#import <SFDatabase.h>
#import <SFDataWrapper.h>
#import <SFDynamicString.h>
#import <SFFiscalPeriodManager.h>
#import "SFVariable.h"
#import "SFSystemProcedure.h"
#import "SFSummaryRangeFunctions.h"
#import "SFStringTableManager.h"
#import "SFParameterData.h"
#import "SFPeriodData.h"
#import "SFFiscalPeriod.h"
#import "NSStringAdditionalMethods.h"

/*
**	Function subtypes
*/
#define  CA				    0		/* Credit amount */
#define  DA				    1		/* Debit amount */

static NSString *descriptionText = @"\n\nThe function takes the following input parameters:\n"
					@"<GL Prefix> - specific number or a search key that uses "
					@"standard SQL keywords(\"like\", \"not like\", \"between\" etc.)\n"
					@"<GL Account> - specific number or Account List name\n"
					@"<SL Account> - subledger account number, or blank\n"
					@"<Document Type> - can be \"Actual\" or \"Budget\"\n"
					@"<Document Class> - can be \"Regualr\" or \"Restatement\"\n"
					@"<Starting Date> - starting date of the time interval for summation\n"
					@"<Ending Date> - ending date of the time interval for summation\n"
					@"<System State> - number of the system state to be \"visible\" to the "
					@"function\n"
					@"<Simulate Posting> = \"Yes\" to simulate posting of all unposted "
					@"documents, \"No\" to honor only actually posted documents";
/*
**	Record ID structure
*/
typedef struct
{
	int accountNumber;		/* Internal GL account number */
	int subledgerNumber;		/* Internal SL account number */
	double credit;				/* Credit amount */
	double debit;				/* Debit amount */
	char docType;			/* Document type */
	char restatement;			/* Document class */
}	SUMMARY_RECORD;

@implementation SFSummaryRangeFunctions : SFSummaryFunctions

- initFor: (id) sender
{
	SFParameterData *parameter;
	id dataWrapper;
	self = [super  initFor:sender];
	[functionNameList  removeAllObjects];
	dataWrapper = [[SFDataWrapper  alloc]  initWithNSString:@"CreditAmount"];
	[functionNameList  addObject:dataWrapper];
	dataWrapper = [[SFDataWrapper  alloc]  initWithNSString:@"DebitAmount"];
	[functionNameList  addObject:dataWrapper];
	parameter = [[SFParameterData  alloc]
						initWithName:@"Starting date"
						dataType:DATE
						displayedDataType:DATE
						ioFlag:INPUT
						runTime:YES
						isSwitchDisplayed:YES];
	[parameterList  insertObject:parameter atIndex:5];
	[parameter  release];
	lastStartingDate = nil;
	return self;
}

- (void) setFunctionName: (NSString *) name
{
	functionName = [name  preserve];
	if (stringsAreEqual(name,@"CreditAmount"))
		functionType = CA;
	else if (stringsAreEqual(name,@"DebitAmount"))
		functionType = DA;
}

- (NSString *) descriptionForFunction: (NSString *) name
{
	if (stringsAreEqual(name,@"CreditAmount"))
		[description  copyString:@"CreditAmount function calculates Credit Amount "
							@"for the specified accounts (see parameters below) "
							@"accumulated since inception of the system."];
	else 	if (stringsAreEqual(name,@"DebitAmount"))
		[description  copyString:@"DebitAmount function calculates Debit Amount "
							@"for the specified accounts (see parameters below) "
							@"accumulated since inception of the system."];
	else
		[description  copyString:@""];
	[description  concatenate:descriptionText];
	return [description  string];
}

- (NSArray *) functionNameList
{
	return (functionNameList);
}

- execute: (NSArray *) parmList
{
	NSString *listName, *prefix, *subledger, *docType, *docClass, *endingDate, *startingDate,
		*posting;
	char class, type;
	short state;
	double amount;
	void *value;
	BOOL sPost;
	[super  execute:nil];
	fiscalPeriods = [[SFFiscalPeriodManager  alloc]  init];
	prefix = [[parmList  objectAtIndex:0]  asciiValue];
	listName = [[parmList  objectAtIndex:1]  asciiValue];
	subledger = [[parmList  objectAtIndex:2]  asciiValue];
	docType = [[parmList  objectAtIndex:3]  asciiValue];
	docClass = [[parmList  objectAtIndex:4]  asciiValue];
	startingDate = [[parmList  objectAtIndex:5]  asciiValue];
	endingDate = [[parmList  objectAtIndex:6]  asciiValue];
	value = [[parmList  objectAtIndex:7]  value];
	if (value != NULL)
		state = (short) [[parmList  objectAtIndex:7]  intValue];
	else
		state = [self  currentSystemState];
	posting = [[parmList  objectAtIndex:8]  asciiValue];
/*
**	Set Document Class to required format
*/
	if (stringsAreEqual(docClass,@"Regular"))
		class = ' ';
	else if (stringsAreEqual(docClass,@"Restatement"))
		class = 'R';
	else
	{
/*
**		Error
*/
		if (NSApp != nil)
			NSRunAlertPanel(@"Run-time error",@"Parameter \"Document Class\" in function "
						       @"\"%@\" has been given an invalid value (\"%@\"). Valid "
						       @"classes are: \"Regular\" or \"Restatement\".",
						       @"OK", nil, nil, functionName,docClass);
		else
		{
/*
**			Background job: quit with error message (DataBase "failure" method
**			will send the message text to the main Server and quit the task).
*/
			NSString *message;
			message = [NSString  stringWithFormat:@"Parameter \"Document Class\" in function "
							@"\"%@\" has been given an invalid value (\"%@\"). Valid "
							@"classes are: \"Regular\" or \"Restatement\".",
							functionName,docClass];
			[database  failure:message];
		}
		return nil;
	}
/*
**	Set Document Type to required format
*/
	if (stringsAreEqual(docType,@"Actual"))
		type = 'A';
	else if (stringsAreEqual(docType,@"Budget"))
		type = 'B';
	else
	{
/*
**		Error
*/
		if (NSApp != nil)
			NSRunAlertPanel(@"Run-time error",@"Parameter \"Document Type\" in function "
						       @"\"%@\" has been given an invalid value (\"%@\"). Valid types "
						       @"are: \"Actual\" or \"Budget\".",
						       @"OK", nil, nil, functionName,docType);
		else
		{
/*
**			Background job: quit with error message (DataBase "failure" method
**			will send the message text to the main Server and quit the task).
*/
			NSString *message;
			message = [NSString  stringWithFormat:@"Parameter \"Document Type\" in function "
							@"\"%@\" has been given an invalid value (\"%@\"). Valid types "
							@"are: \"Actual\" or \"Budget\".",functionName,docType];
			[database  failure:message];
		}
		return nil;
	}
/*
**	Set Posting status to required format
*/
	if ([posting  caseInsensitiveCompare:@"Yes"] == NSOrderedSame)
		sPost = YES;
	else if ([posting  caseInsensitiveCompare:@"No"] == NSOrderedSame)
		sPost = NO;
	else
	{
/*
**		Error
*/
		if (NSApp != nil)
			NSRunAlertPanel(@"Run-time error",@"Parameter \"Simulate Posting\" in function "
						       @"\"%@\" has been given an invalid value (\"%@\"). Valid "
						       @"values are: \"Yes\" or \"No\".",
						       @"OK", nil, nil, functionName,posting);
		else
		{
/*
**			Background job: quit with error message (DataBase "failure" method
**			will send the message text to the main Server and quit the task).
*/
			NSString *message;
			message = [NSString  stringWithFormat:@"Parameter \"Simulate Posting\" in "
							@"function \"%@\" has been given an invalid value (\"%@\"). "
							@"Valid values are: \"Yes\" or \"No\".",functionName,posting];
			[database  failure:message];
		}
		return nil;
	}
/*
**	Execute appropriate function
*/
	switch (functionType)
	{
		case CA:
			amount = [self  creditAmountForList:listName
						glPrefix:prefix
						subledger:subledger
						documentType:type
						documentClass:class
						startingDate:startingDate
						endingDate:endingDate
						systemState:state
						simulatePosting:sPost];
			break;
		case DA:
			amount = [self  debitAmountForList:listName
						glPrefix:prefix
						subledger:subledger
						documentType:type
						documentClass:class
						startingDate:startingDate
						endingDate:endingDate
						systemState:state
						simulatePosting:sPost];
			break;
	}
	[returnVariable  setValue:&amount];
	return (returnVariable);
}

- (void) extractDataForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledgerNumber: (int) slNumber
	startingDate: (NSString *) startingDate
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
/*
**	Get totals from TXSUMMARY table for the specified list of GL accounts
**	(or a single account, passed in place of "listName"). The table is
**	searched for the period the ending date belongs to; if the ending date
**	does not fall on the last day of the period, the amounts are adjusted from
**	the Transaction Detail table.
*/
	id periodList;
	int i, count, j, recordsCount, accountNumber, subledgerNumber,
		startingCorrection, endingCorrection;
	short systemState, startingFiscalPeriod, endingFiscalPeriod;
	double creditAmount, debitAmount, amount;
	NSString *buffer, *table, *operator, *endStartingPeriodDate = nil, *startEndingPeriodDate = nil;
	char docType, restatement;
	BOOL fullStartingPeriod, fullEndingPeriod, found, slFlag;
	SFPeriodData *periodData;
	SFFiscalPeriod *fiscalData;
	SUMMARY_RECORD record, *recordPtr;
/*
**	If nothing has changed since the last request, skip everything
*/
	slFlag = (slNumber != 0);
	if (lastListName != NULL)
	{
		if (stringsAreEqual(listName,lastListName) &&
			stringsAreEqual(prefix,lastPrefix) &&
			stringsAreEqual(endingDate,lastEndingDate) &&
			stringsAreEqual(startingDate,lastStartingDate) &&
			state == lastState &&
			slFlag == lastSubledgerFlag &&
			slNumber == lastSLAccount &&
			sPost == lastPostingFlag)
			return;
	}
	lastSubledgerFlag = slFlag;
/*
**	Get the list of accounts
*/
	[records  empty];
	[self  getAccountsFromList:listName forGlPrefix:prefix];
	if ([accounts  count] == 0)
		return;
/*
**	Get the accounting period of the starting date
*/
	periodData = [fiscalPeriods  periodForDate:startingDate];
	if (periodData == nil)
	{
/*
**		Date is outside the fiscal period range. If it is after the end
**		of the last fiscal period, quit.
*/
		periodList = [fiscalPeriods  periodList];
		count = [periodList  count];
		if (count == 0)
			return;
		fiscalData = [periodList  objectAtIndex:count - 1];
		if ([startingDate  compare:[fiscalData  endDate]] == NSOrderedDescending)
			return;
		count = [periodList  count];
		if (count == 0)
			return;
		fiscalData = [periodList  objectAtIndex:0];
		startingDate = [fiscalData  startDate];
		periodData = [fiscalPeriods  periodForDate:startingDate];
	}
	startingFiscalPeriod = [periodData  internal];
	fullStartingPeriod = stringsAreEqual([periodData  startDate],startingDate);
	endStartingPeriodDate = [periodData  endDate];
	if (fullStartingPeriod == NO)
		startingCorrection = 1;
	else
		startingCorrection = 0;
/*
**	Get the accounting period of the ending date
*/
	periodData = [fiscalPeriods  periodForDate:endingDate];
	if (periodData == nil)
	{
/*
**		Date is outside the fiscal period range. If it is prior to the end
**		of the first fiscal period, quit.
*/
		periodList = [fiscalPeriods  periodList];
		count = [periodList  count];
		if (count == 0)
			return;
		fiscalData = [periodList  objectAtIndex:0];
		if ([endingDate  compare:[fiscalData  endDate]] == NSOrderedAscending)
			return;
		fiscalData = [periodList  objectAtIndex:count - 1];
		endingDate = [fiscalData  endDate];
		periodData = [fiscalPeriods  periodForDate:endingDate];
	}
	endingFiscalPeriod = [periodData  internal];
	fullEndingPeriod = stringsAreEqual([periodData  endDate],endingDate);
	startEndingPeriodDate = [periodData  startDate];
	if (fullEndingPeriod == NO)
		endingCorrection = 1;
	else
		endingCorrection = 0;
/*
**	If simulated posting is requested, do it here
*/
	if (sPost)
	{
		[self  simulatePostingBetweenPeriod:startingFiscalPeriod
			and:endingFiscalPeriod
			subledgerNumber:slNumber
			startingDate:startingDate
			endingDate:endingDate];
	}
/*
**	Read data for each account, based on the latest 
**	available period (relative to the ending date) and state.
*/
	count = [accounts  count];
	table = [NSString  stringWithFormat:@"%@ A",TXSUMMARY];
	if (slFlag)
	{
		if (count == 1)
		{
			operator = @"=";
			subledgerNumber = slNumber;
		}
		else
		{
			operator = @"!=";
			subledgerNumber = 0;
		}
	}
	else
	{
		operator = @"=";
		subledgerNumber = 0;
	}
	for (i = 0; i < count; i++)
	{
		accountNumber = *((int *) [accounts  elementAt:i]);
		buffer = [NSString  stringWithFormat:@"%d",accountNumber];
		[searchKey  sprintf:@"where SF_IDG = %d and SF_IDS %@ %d and "
						@"SF_SystemState <= %d and SF_AccountingPeriod >= %d and "
						@"SF_AccountingPeriod <= %d",
						accountNumber, operator,subledgerNumber,state,
						startingFiscalPeriod + startingCorrection,
						endingFiscalPeriod - endingCorrection];
		[database  getRowsFor:@"SF_IDS,SF_CreditAmount,SF_DebitAmount,"
						        @"SF_SystemState,SF_DocType,SF_Restatement"
				  fromTable:table
				  conditions:[searchKey  string]
				  convertAs:@"INT,MONEY,MONEY,SMALL,CHAR,CHAR"
				  intoVars:&subledgerNumber,&creditAmount,&debitAmount,
				      		 &systemState,&docType,&restatement];
		while ([database  getNextRow] == SUCCEED)
		{
/*
**			Try to update record in Storage (assuming it exists)
*/
			systemState = state;
			recordsCount = [records  count];
			for (j = 0, found = NO; j < recordsCount; j++)
			{
				recordPtr = (SUMMARY_RECORD *) [records  elementAt:j];
				if (recordPtr->accountNumber == accountNumber &&
					recordPtr->subledgerNumber == subledgerNumber &&
					recordPtr->docType == docType &&
					recordPtr->restatement == restatement)
				{
/*
**					Record exists - update it
*/
					recordPtr->credit += creditAmount;
					recordPtr->debit += debitAmount;
					found = YES;
					break;
				}
			}
			if (found == NO)
			{
/*
**				Record does not exis - add it
*/
				record.accountNumber = accountNumber;
				record.subledgerNumber = subledgerNumber;
				record.credit = creditAmount;
				record.debit = debitAmount;
				record.docType = docType;
				record.restatement = restatement;
				[records  addElement:&record];
			}
		}
	}
	if (fullStartingPeriod == NO || fullEndingPeriod == NO)
	{
/*
**		Add missing data from TRANSACTION table
*/
		recordPtr = (SUMMARY_RECORD *) [records  elementAt:i];
		if (count == 1 && slNumber != 0)
			[searchKey  sprintf:@"where SF_SystemState <= %d and SF_Posted = 'P' and "
							@"SF_IDS = %d and SF_IDG in (",state,slNumber];
		else
			[searchKey  sprintf:@"where SF_SystemState <= %d and SF_Posted = 'P' and "
							@"SF_IDG in (",state];
		count = [accounts  count];
		for (j = 0; j < count; j++)
		{
			accountNumber = *((int *) [accounts  elementAt:j]);
			buffer = [NSString  stringWithFormat:@"%d",accountNumber];
			if (j) [searchKey  concatenate:@","];
			[searchKey  concatenate:buffer];
		}
		[searchKey  concatenate:@")"];
		if (fullStartingPeriod == NO && fullEndingPeriod == NO)
		{
			if (startingFiscalPeriod != endingFiscalPeriod)
				[description  sprintf:@" and ((SF_AccountingPeriod = %d and "
								@"SF_DocumentDate >= '%@' and SF_DocumentDate <= "
								@"'%@') or (SF_AccountingPeriod = %d and "
								@"SF_DocumentDate >= '%@' and SF_DocumentDate <= "
								@"'%@'))",startingFiscalPeriod,startingDate,
								endStartingPeriodDate,endingFiscalPeriod,
								startEndingPeriodDate,endingDate];
			else
				[description  sprintf:@" and (SF_AccountingPeriod = %d and "
								@"SF_DocumentDate >= '%@' and SF_DocumentDate <= "
								@"'%@')",startingFiscalPeriod,startingDate,endingDate];
		}
		else if (fullStartingPeriod)
		{
			[description  sprintf:@" and (SF_AccountingPeriod = %d and SF_DocumentDate >= "
							@"'%@' and SF_DocumentDate <= '%@')",endingFiscalPeriod,
							startEndingPeriodDate,endingDate];
		}
		else if (fullEndingPeriod)
		{
			[description  sprintf:@" and (SF_AccountingPeriod = %d and SF_DocumentDate >= "
							@"'%@' and SF_DocumentDate <= '%@')",startingFiscalPeriod,
							startingDate,endStartingPeriodDate];
		}
		[searchKey  concatenate:[description  string]];
		[database  getRowsFor:@"SF_IDG,SF_IDS,SF_Amount,SF_DocType,SF_Restatement"
				  fromTable:TRANSACTION
				  conditions:[searchKey  string]
				  convertAs:@"INT,INT,MONEY,CHAR,CHAR"
				  intoVars:&accountNumber,&subledgerNumber,&amount,&docType,
				      		 &restatement];
		while ([database  getNextRow] == SUCCEED)
		{
			if (slFlag == NO)
				subledgerNumber = 0;
			if (amount > 0.0)
			{
				debitAmount = amount;
				creditAmount = 0.0;
			}
			else
			{
				debitAmount = 0.0;
				creditAmount = amount;
			}
/*
**			Try to update record (assuming it exists)
*/
			count = [records  count];
			for (j = 0, found = NO; j < count; j++)
			{
				recordPtr = (SUMMARY_RECORD *) [records  elementAt:j];
				if (recordPtr->accountNumber == accountNumber &&
					recordPtr->subledgerNumber == subledgerNumber &&
					recordPtr->docType == docType &&
					recordPtr->restatement == restatement)
				{
/*
**					Record exists - update it
*/
					recordPtr->credit += creditAmount;
					recordPtr->debit += debitAmount;
					found = YES;
					break;
				}
			}
			if (found == NO)
			{
/*
**				Record does not exist - add it
*/
				record.accountNumber = accountNumber;
				record.subledgerNumber = subledgerNumber;
				record.credit = creditAmount;
				record.debit = debitAmount;
				record.docType = docType;
				record.restatement = restatement;
				[records  addElement:&record];
			}
		}
	}
}

- (void) simulatePostingBetweenPeriod: (short) startingFiscalPeriod
	and: (short) endingFiscalPeriod
	subledgerNumber: (int) slNumber
	startingDate: (NSString *) startingDate
	endingDate: (NSString *) endingDate
{
/*
**	Post all completed (but not yet posted) documents from
**	TRANSACTION table into Storage
*/
	int accountNumber, subledgerNumber, i, count;
	char docType, docRestatement;
	NSString *buffer;
	double amount, debitAmount, creditAmount, ytdIncrement;
	BOOL found;
	SUMMARY_RECORD record, *recordPtr;
	count = [accounts  count];
	if (count == 1 && slNumber != 0)
		[searchKey  sprintf:@"where ((SF_AccountingPeriod >= %d and SF_AccountingPeriod "
						@"<= %d) and (SF_DocumentDate >= '%@' and SF_DocumentDate "
						@"<= '%@')) and SF_Posted = 'C' and SF_IDS = %d and SF_IDG in (",
						startingFiscalPeriod,endingFiscalPeriod,startingDate,endingDate,						slNumber];
	else
		[searchKey  sprintf:@"where ((SF_AccountingPeriod >= %d and SF_AccountingPeriod "
						@"<= %d) and (SF_DocumentDate >= '%@' and SF_DocumentDate "
						@"<= '%@')) and SF_Posted = 'C' and SF_IDG  in (",
						startingFiscalPeriod,endingFiscalPeriod,startingDate,endingDate];
/*
**	Set the list of accounts to search
*/
	for (i = 0; i < count; i++)
	{
		accountNumber = *((int *) [accounts  elementAt:i]);
		buffer = [NSString  stringWithFormat:@"%d",accountNumber];
		if (i) [searchKey  concatenate:@","];
		[searchKey  concatenate:buffer];
	}
	[searchKey  concatenate:@")"];
/*
**	Search the table
*/
	[database  getRowsFor:@"SF_IDG,SF_IDS,SF_Amount,SF_DocType,SF_Restatement"
			  fromTable:TRANSACTION
			  conditions:[searchKey  string]
			  convertAs:@"INT,INT,MONEY,CHAR,CHAR"
			  intoVars:&accountNumber,&subledgerNumber,&amount,
					 &docType,&docRestatement];
	while ([database  getNextRow] == SUCCEED)
	{
		if (slNumber == 0)
			subledgerNumber = 0;
		ytdIncrement = amount;
		if (amount > 0.0)
		{
			debitAmount = amount;
			creditAmount = 0.0;
		}
		else
		{
			debitAmount = 0.0;
			creditAmount = amount;
		}
/*
**		Try to update record (assuming it exists)
*/
		count = [records  count];
		for (i = 0, found = NO; i < count; i++)
		{
			recordPtr = (SUMMARY_RECORD *) [records  elementAt:i];
			if (recordPtr->accountNumber == accountNumber &&
				recordPtr->subledgerNumber == subledgerNumber &&
				recordPtr->docType == docType &&
				recordPtr->restatement == docRestatement)
			{
/*
**				Record exists - update it
*/
				recordPtr->credit += creditAmount;
				recordPtr->debit += debitAmount;
				found = YES;
				break;
			}
		}
		if (found == NO)
		{
/*
**			Record does not exis - add it
*/
			record.accountNumber = accountNumber;
			record.subledgerNumber = subledgerNumber;
			record.credit = creditAmount;
			record.debit = debitAmount;
			record.docType = docType;
			record.restatement = docRestatement;
			[records  addElement:&record];
		}
	}	
}

- (void) prepareAllDataForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	startingDate: (NSString *) startingDate
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
	int accountNumber, subledgerAccount;
	NSString *subledgerName;
	unichar c;
	BOOL subledgerFlag, isDigit;
/*
**	Next we will try to find the GL and SL account numbers.
**	This can be done only if the "listName" is a single GL account,
**	and GL prefix does not have any wildcards.
*/
	subledgerFlag = !isEmptyString(subledger);
	if (isEmptyString(listName))
		isDigit = NO;
	else
	{
		c = [listName  characterAtIndex:0];
		isDigit = [decimalDigitSet  characterIsMember:c];
	}
	if (isDigit && [prefix  isInteger])
	{
		accountNumber = lastGLAccount;
		if (!stringsAreEqual(listName,lastListName) || !stringsAreEqual(prefix,lastPrefix))
		{
/*
**			Find the name of the Subledger Master
*/
			subledgerName = nil;
			[masterName  release];
			masterName = nil;
			if (isPrefixUsed)
				[searchKey  sprintf:@"where SF_GLPrefix = '%@' and SF_GLNumber = '%@'",
								prefix,listName];
			else
				[searchKey  sprintf:@"where SF_GLNumber = '%@'",listName];
			[database  getRowsFor:@"SF_IDG,SF_Subledger"
					   fromTable:VALIDACCOUNT
					   conditions:[searchKey  string]
					   convertAs:@"INT,STRING"
					   intoVars:&accountNumber,&subledgerName];
			while ([database  getNextRow] == SUCCEED)
				subledgerName = [subledgerName  truncatedString];
			if (!isEmptyString(subledgerName))
				masterName = [[NSString  alloc]
					initWithFormat:@"SF_MV%@",subledgerName];
		}
/*
**		If the subledger is given, find its internal number (this can be done
**		only if the "listName" is a single GL account, and GL prefix does
**		not have any wildcards. In other words, we can only deal with a
**		concrete Subledger Master, attached to a known GL Number).
*/
		if (subledgerFlag == YES)
		{
			if (!isEmptyString(masterName) && (!stringsAreEqual(subledger,lastSubledger)
				|| accountNumber != lastGLAccount))
			{
				[lastSubledger  release];
				lastSubledger = [subledger  preserve];
/*
**				Find internal number of the given subledger number
*/
				lastSLAccount = 0;
				[searchKey  sprintf:@"where SF_SubledgerNumber = '%@' and SF_IDG = %d",
								subledger,accountNumber];
				[database  getRowsFor:@"SF_ID"
						   fromTable:masterName
						   conditions:[searchKey  string]
						   convertAs:@"INT"
						   intoVars:&subledgerAccount];
				while ([database  getNextRow] == SUCCEED);
			}
			else
				subledgerAccount = lastSLAccount;
		}
		else
			subledgerAccount = 0;
		lastGLAccount = accountNumber;
	}
	else
	{
		lastGLAccount = -1;
		subledgerAccount = 0;
	}
/*
**	Get all summary values for the given list of GL Accounts
**	(and subledger, if it is not NULL), up to the given ending date.
*/
	[self  extractDataForList:listName
		 glPrefix:prefix
		 subledgerNumber:subledgerAccount
		 startingDate:startingDate
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
/*
**	Save parameters for subsequent testing
*/
	lastSLAccount = subledgerAccount;
	[self  saveList:listName
		 glPrefix:prefix
		 startingDate:startingDate
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
}

- (void) saveList: (NSString *) listName
	glPrefix: (NSString *) prefix
	startingDate: (NSString *) startingDate
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
	[super  saveList:listName
		     glPrefix:prefix
		     endingDate:endingDate
		     systemState:state
		     simulatePosting:sPost];
	[lastStartingDate  release];
	lastStartingDate = [startingDate  preserve];
}

- (double) creditAmountForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	startingDate: (NSString *) startingDate
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
/*
**	Return Credit amount for the given list of GL Accounts
**	(and subledger, if it is not NULL), up to the given ending date.
*/
	int i, count;
	double total;
	SUMMARY_RECORD *record;
	[self  prepareAllDataForList:listName
		 glPrefix:prefix
		 subledger:subledger
		 documentType:docType
		 documentClass:restatement
		 startingDate:startingDate
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
/*
**	Calculate Credit amount for the given GL account
**	(and subledger, if required), observing the required type and
**	class of the documents.
*/
	total = 0.0;
	count = [records  count];
	for (i = 0; i < count; i++)
	{
		record = (SUMMARY_RECORD *) [records  elementAt:i];
		if (lastGLAccount != -1 && record->accountNumber != lastGLAccount)
			continue;
		if (record->subledgerNumber != lastSLAccount)
			continue;
		if (record->docType != docType)
			continue;
		if (restatement != 'B')
		{
			if (record->restatement != restatement)
				continue;
		}
		total += record->credit;
	}
	return (total);
}

- (double) debitAmountForList: (NSString *) listName
	glPrefix: (NSString *) prefix
	subledger: (NSString *) subledger
	documentType: (char) docType
	documentClass: (char) restatement
	startingDate: (NSString *) startingDate
	endingDate: (NSString *) endingDate
	systemState: (short) state
	simulatePosting: (BOOL) sPost
{
/*
**	Return Debit amount for the given list of GL Accounts
**	(and subledger, if it is not NULL), up to the given ending date.
*/
	int i, count;
	double total;
	SUMMARY_RECORD *record;
	[self  prepareAllDataForList:listName
		 glPrefix:prefix
		 subledger:subledger
		 documentType:docType
		 documentClass:restatement
		 startingDate:startingDate
		 endingDate:endingDate
		 systemState:state
		 simulatePosting:sPost];
/*
**	Calculate Debit amount for the given GL account
**	(and subledger, if required), observing the required type and
**	class of the documents.
*/
	total = 0.0;
	count = [records  count];
	for (i = 0; i < count; i++)
	{
		record = (SUMMARY_RECORD *) [records  elementAt:i];
		if (lastGLAccount != -1 && record->accountNumber != lastGLAccount)
			continue;
		if (record->subledgerNumber != lastSLAccount)
			continue;
		if (record->docType != docType)
			continue;
		if (restatement != 'B')
		{
			if (record->restatement != restatement)
				continue;
		}
		total += record->debit;
	}
	return (total);
}

- (void) dealloc
{
	[lastStartingDate  release];
	[super  dealloc];
}

@end