/*
**	SFDataWrapper.h
**
**	by:   Alex Molochnikov
**
**	Release 2.0
**	Copyright (c) 1998, Phoenix Data Trend.  All rights reserved.
*/

#import <AppKit/AppKit.h>
#import <Foundation/NSPortCoder.h>

@interface SFDataWrapper : NSObject
/*
**	A SFDataWrapper is an NSObject that holds data
*/
{
	int dataType;					/* Type of data */
	int dataSize;					/* Size of data */
	void *data;					/* Data buffer */
	union
	{
		double doubleValue;
		float floatValue;
		int intValue;
		short shortValue;
		BOOL boolValue;
		unsigned char tinyValue;
	} fixedBuffer;					/* Buffer for alignement-sensitive data */
	struct _dataWrapperFlags
	{
		unsigned int isNull:1;		/* Whether the carried data is to be treated as NULL */
		unsigned int restricted:1;		/* Marks whether the data has been edited out or not */
		unsigned int isObject:1;		/* Whether the carried data is an NSObject */
		unsigned int isMalloced:1;	/* If value points to malloc-allocated buffer */
		unsigned int reserved:4;		/* Reserved */
	} dataWrapperFlags;
}

+ (void) initialize;
+ (void) setRunsInServer: (BOOL) flag;
- init;
- initWithData: (const void *) dataPtr
	type: (int) type;
- initWithUnterminatedString: (const void *) dataPtr
	size: (int) size;
- initWithNSString: (NSString *) string;
- initWithNSString: (NSString *) string type: (int) type;
- initForType: (int) type
	maxSize: (int) size;
- initFromQuestion: (id) question;
- initFromVariable: (id) variable;
- initFromObject: (id) object;
- (void) setObject: (id) object;
- copy;
- copyWithZone: (NSZone *) zone;
- (void) setDataType: (int) type;
- (void) setContentsFromVariable: (id) variable;
- (int) fieldType;
- (void) setSize: (int) size;
- (void) setData: (const void *) dataPtr;
- (void) setNSData: (const void *) dataPtr;
- (NSData *) nsData;
- (void) setNull: (BOOL) flag;
- (void) instantiateObject;
- (int) dataType;
- (int) dataSize;
- (void *) data;
- (void *) rawData;
- (NSObject *) object;
- (NSString *) string;
- (int) intValue;
- (short) shortValue;
- (unsigned char) tinyValue;
- (char) charValue;
- (float) floatValue;
- (double) doubleValue;
- (BOOL) boolValue;
- (BOOL) isNull;
- (BOOL) isEmpty;
- (BOOL) isObject;
- (void *) dataBuffer;
- (NSComparisonResult) compareDataWrapper: (SFDataWrapper *) dataWrapper;
- (void) setDataRestrictionEnabled: (BOOL) enabled;
- (BOOL) isDataRestrictionEnabled;
- (void) dealloc;
- (void) freeDataBuffer;
- (NSData *) nsDataEncoding;
- initWithCoder: (NSCoder *) aDecoder;
- (void) encodeWithCoder: (NSCoder *) aCoder;
- replacementObjectForPortCoder: (NSPortCoder *) encoder;

@end