/*
**	SFDatabase.h
**
**	by:   Alex Molochnikov
**
**	Release 2.0
**	Copyright (c) 1998, Phoenix Data Trend.  All rights reserved.
*/

#import <Foundation/NSObject.h>
#import "SFDataServer.h"
#import <SFStorage.h>
#import <SFDynamicString.h>
#import "dbconstants.h"

@interface SFDataBase : NSObject
/*
**	A SFDataBase is an Object that provides an interface to the SFDataServer
**	object locally or remotely
*/
{
	SFDataServer *database;				/* SFDataServer object - proxy or copy */
	SFStorage *pointerList;					/* Storage for variable pointers */
	NSMutableArray *dataContainer;			/* NSMutableArray to hold DataWrappers */
	id mainServer;							/* Proxy to the Main Server */
	id client;								/* Client object */
	SFStorage *nullArray;					/* Array of NULL indexes */
	NSString *application;					/* Application name */
	NSString *user;						/* User ID */
	NSString *password;					/* User password */
	NSString *localHostName;				/* Local host name */
	NSString *signature;						/* Signature of background process */
	NSCharacterSet *separatorSet;			/* Set of separator chars for format strings */
	BOOL isRemoteConnection;				/* YES if SFDataServer is passed by proxy */
	BOOL showErrorMessage;				/* Whether to display error message */
	BOOL isCopy;							/* Whether the object is a copy */
	SFDynamicString *editedFormatString;		/* New format string */
	id securityManager;						/* SecurityManager object */
	int securityLevel;						/* Degree of Security */
	BOOL securityCheck;					/* Security check flag */
	NSString *currentUser;					/* Current user for security purposes */
}

+ (void) initialize;
+ (void) setDebugging: (BOOL) flag;
+ (void) setAppName: (NSString *) name;
+ (BOOL) restrictedKeyword: (NSString *) string;
+ (void) translateDataTypesIn: (SFDynamicString *) dynamicString;
+ (void) tokeniseDataTypesIn: (SFDynamicString *) dynamicString;
+ (NSString *) dataFormatFor: (int) type;
+ (int) maxVarCharSize;
+ (NSString *) illegalCharacters;
+ (NSString *) concatenationOperator;
+ (NSString *) moneyPrefix;
+ (NSString *) booleanYes;
+ (NSString *) booleanNo;
+ (NSString *) isNullClause;
+ (NSString *) isNotNullClause;
+ (NSString *) createDatabaseCommand;
+ (NSString *) stringToConvertToFloat: (NSString *) fieldName;
+ (BOOL) isCaseSensitiveSearchPossible;
- initForUser: (NSString *) uid
	  password: (NSString *) pwd
   	  appName: (NSString *) app;
- initForUser: (NSString *) uid
	  password: (NSString *) pwd
   	  appName: (NSString *) app
   	  signature: (NSString *) sig
	  isBackgroundProcess: (BOOL) backgroundProcess;
- initBlindly;
- copy;
- mainServer;
- client;
- (void) registerObserver: (id) sender selector: (SEL) action;
- (void) unregisterObserver: (id) sender;
- (void) setDefaultHost: (NSString *) host;
- (int) getAvailableDatabasesIntoContainer: (NSMutableArray **) container;
- (int) beginTransaction;
- (int) beginTransactionWithMetadata;
- (int) commitTransaction;
- (int) commitTransactionWithMetadata;
- (int) rollbackTransaction;
- (int) rollbackTransactionWithMetadata;
- (int) useDatabase: (NSString *) databaseName;
- (NSString *) currentDatabase;
- (NSString *) createIndexExtention;
- (NSString *) hostName;
- (NSString *) processId;
- (NSString *) lastSQLCommand;
- (void) setDisplayErrorMessageEnabled: (BOOL) flag;
- (void) displayErrorMessage;
- (NSString *) errorMessage;
- (int) errorNumber;
- (void) setSecurityCheckEnabled: (BOOL) flag;
- (BOOL) isSecurityCheckEnabled;
- (NSString *) nameOfObjectingIndex;
- (void) dealloc;
- (int) getForeignDataIntoContainer: (NSMutableArray **) container;
- (int) getRowsFor: (NSString *) columnNames
	  fromTable: (NSString *) tableName
	  conditions: (NSString *) searchCondition
	  convertAs: (NSString *) formatString
	  intoContainer: (NSMutableArray **) container;
- (int) getRowsFor: (NSString *) columnNames
	  fromTable: (NSString *) tableName
	  conditions: (NSString *) searchCondition
	  convertAs: (NSString *) formatString;
- (int) getNextRowIntoContainer: (NSMutableArray **) container;
- (int) getRowsFor: (NSString *) columnNames
	  fromTable: (NSString *) tableName
	  conditions: (NSString *) searchCondition
	  convertAs: (NSString *) formatString
	  intoVars: (void *) var, ...;
- (int) getRowsFor: (NSString *) columnNames
	  fromTable: (NSString *) tableName
	  conditions: (NSString *) searchCondition
	  convertAs: (NSString *) formatString
	  intoList: (void **) list;
- (int) getNextRow;
- (int) addRowFor: (NSString *) columnNames
	  inTable: (NSString *) tableName
	  convertAs: (NSString *) formatString
	  fromVars: (void *) var, ...;
- (int) updateRowsFor: (NSString *) columnNames
	  inTable: (NSString *) tableName
	  where: (NSString *) searchCondition
	  convertAs: (NSString *) formatString
	  fromVars: (void *) var, ...;
- (int) setExclusiveLockOnTable: (NSString *) tableName;
- (int) existsRow: (NSString *) searchCondition
	  inTable: (NSString *) tableName;
- (int) countRows: (int *) count
	  inTable: (NSString *) tableName
	  where: (NSString *) searchCondition;
- (int) updateRowsFor: (NSString *) columnNames
	  inTable: (NSString *) tableName
	  where: (NSString *) searchCondition
	  convertAs: (NSString *) formatString
	  fromContainer: (id) container;
- (int) addRowFor: (NSString *) columnNames
	  inTable: (NSString *) tableName
	  convertAs: (NSString *) formatString
	  fromContainer: (id) container;
- (int) deleteRowFromTable: (NSString *) tableName
	  where: (NSString *) searchCondition;
- (int) execSQL: (NSString *) sqlStatement;
- (void) getCurrentDate: (NSString **) date time: (NSString **) time;
- (int) renameTable: (NSString *) oldName to: (NSString *) newName;
- (int) renameColumn: (NSString *) oldName to: (NSString *) newName
	  inTable: (NSString *) tableName;
- (int) alterTable: (NSString *) tableName
	addFields: (NSString *) addString
	dataTypes: (NSString *) typeString
	dropFields: (NSString *) dropString;
- (int) existsTable: (NSString *) tableName;
- (int) affectedRowCount;
- (NSString *) caseInsensitiveStringFor: (NSString *) string;
- (int) createDatabase: (NSString *) command;
- (int) dropDatabase: (NSString *) dbName;
- (int) cleanUpDatabase: (NSString *) dbName regardlessOfFlag: (BOOL) flag;
- (int) cleanUpDatabase: (NSString *) dbName;
- (void) checkFormatString: (NSString *) formatString
	withColumnString: (NSString *) columnString
	table: (NSString *) tableName
	forAccessType: (int) accessType;
- (int) getAccessMaskForColumn: (NSString *) columnName table: (NSString *) tableName;
- (void) securityManagerFreed;
- (void) failure: (NSString *) text;
- (id) replacementObjectForPortCoder: (NSPortCoder *) encoder;

@end
