/*
**	SFVariable.h
**
**	by:   Alex Molochnikov
**
**	Release 2.0
**	Copyright (c) 1998, Phoenix Data Trend.  All rights reserved.
*/

#import <AppKit/AppKit.h>
#import "SFQuestion.h"

/*
**	SFVariable type constants
*/
#define	CONSTANT		0		/* Regular transaction field */
#define	DATAFIELD		1		/* Transaction data field */
#define	TMPVAR			2		/* Temporary variable */
#define	SYSVAR			3		/* System variable */
#define	PARMVAR		4		/* Parameter variable */

@interface SFVariable : NSObject
{
@public
	SFQuestion *delegate;			/* Delegate Question object */
	NSMutableArray *dependentList;	/* List of dependent statements */
	NSMutableArray *definingList;		/* List of defining statements */
	id asciiString;					/* DynamicString for ASCII representation */
	id valueListManager;			/* Value List manager */
	NSString *varName;				/* Name of the variable */
	NSString *systemName;			/* System name of the corresponding field */
	NSString *columnName;			/* Column name of the corresponding field */
	NSString *displayName;			/* Name to be displayed in traceback (DB stmt only) */
	NSString *valueList;				/* Name of attached Value List, if any */
	void *value;					/* Pointer to the value buffer */
	int dataType;					/* Data type of value */
	int dataSize;					/* Size of data buffer */
	unsigned char precision;			/* Precision of numeric data */
	int varType;					/* Type of variable */
	int parameterType;				/* Type of Parameter */
	int idNumber;					/* Id number of the field */
	struct __variableFlags
	{
		unsigned int alwaysReset:1;			/* If it should be reset on every pass */
		unsigned int valueIsObject:1;			/* If IMAGE is stored as NXImage object */
		unsigned int isValueListUserDef:1;		/* If the Value List is user-defined */
		unsigned int isCensoredOut:1;			/* If the contents cannot be shown */
		unsigned int isNull:1;				/* If the contents is null */
		unsigned int isTestingForEqual:1;		/* If in isEqualTo: method */
		unsigned int isMalloced:1;			/* If value points to malloc-allocated buffer */
		unsigned int isEncrypted:1;			/* If contents (TEXT) is encrypted */
	} variableFlags;
	union
	{
		double doubleNumber;		/* Buffer for FLOAT or MONEY value */
		int integerNumber;			/* Buffer for INTEGER value */
		short shortNumber;			/* Buffer for SHORT value */
		unsigned char tinyNumber;	/* Buffer for TINY value */
		id object;					/* Buffer for IMAGE value (object id) */
		BOOL logicalValue;			/* Buffer for LOGICAL value */
	} fixedBuffer;
}

+ (void) initialize;
- initName: (NSString *) name type: (int) type;
- copy;
- (void) addStatementToList: (id) statement;
- (void) addDefiningStatement: (id) statement;
- (void) setDelegate: (id) object;
- (void) setDataType: (int) type;
- (void) setPrecision: (unsigned char) prc;
- (void) setParameterType: (int) type;
- (void) setVariableType: (int) type;
- (void) setFieldNumber: (int) number;
- (void) setSystemName: (NSString *) name;
- (void) setColumnName: (NSString *) name;
- (void) setDisplayName: (NSString *) name;
- (void) setValueList: (NSString *) name isUserDefined: (BOOL) flag;
- (void) zeroOut;
- (void) reset;
- (void) setValue: (void *) data;
- (void) setDataSize: (int) size;
- (int ) dataSize;
- (BOOL) isCensoredOut;
- (BOOL) isEncrypted;
- (BOOL) isNull;
- (void) setValueFrom: (id) sender;
- (void) setValueFromDelegate;
- (BOOL) isValueAnObject;
- (void) markDependentStatements: (id) sender;
- (void) markDefiningStatements: (id) sender;
- (BOOL) dependsOnVariable: (id) variable;
- (BOOL) isEqualTo: (id) variable;
- (BOOL) hasSameDataTypeAs: (id) variable;
- delegate;
- (int) dataType;
- (unsigned char) precision;
- (int) parameterType;
- (int) varType;
- (NSString *) varName;
- (NSString *) systemName;
- (NSString *) columnName;
- (NSString *) displayName;
- (NSString *) valueList;
- (int) idNumber;
- (void *) value;
- (int) intValue;
- (double) doubleValue;
- (BOOL) isEmpty;
- (BOOL) isValueListUserDefined;
- (void *) data;
- (NSString *) asciiValue;
- (NSString *) dbAsciiValue;
- (NSString *) valueFromListFor: (NSString *) key;
- dateFormat;
- (void) releaseValue;
- (void) dealloc;
- initWithCoder: (NSCoder *) aDecoder;
- (void) encodeWithCoder: (NSCoder *) aCoder;
- replacementObjectForPortCoder: (NSPortCoder *) aPortCoder;

@end